@@ -73,6 +73,17 @@ def sort_RGB_gray_to_top(a, b):
 		return 0
 
 
+def sort_RGB_to_top_factory(i1, i2, i3, i4):
+	def sort_RGB_to_top(a, b):
+		if a[i1] == a[i2] and not a[i3] and a[i4]:
+			if b[i1] == b[i2] and not b[i3] and b[i4] and a[i4] > b[i4]:
+				return 1
+			return -1
+		else:
+			return 0
+	return sort_RGB_to_top
+
+
 def sort_RGB_white_to_top(a, b):
 	sum1, sum2 = sum(a[:3]), sum(b[:3])
 	if sum1 == 300:
@@ -186,6 +197,8 @@ class CGATS(dict):
 	
 	datetime = None
 	filename = None
+	fileName = property(lambda self: self.filename,
+						lambda self, filename: setattr(self, "filename", filename))
 	key = None
 	_modified = False
 	mtime = None
@@ -360,13 +373,25 @@ class CGATS(dict):
 	def get_descriptor(self):
 		""" Return descriptor """
 		desc = self.queryv1("DESCRIPTOR")
-		if not desc or desc == "Not specified":
-			desc = self.queryv1("INSTRUMENT")
-			if desc:
-				desc += " & "
+		is_ccss = self.get(0, self).type == "CCSS"
+		if not desc or desc == "Not specified" or is_ccss:
+			if not is_ccss:
+				desc = self.queryv1("INSTRUMENT")
+				if desc:
+					display = self.queryv1("DISPLAY")
+					if display:
+						desc += " & " + display
 			else:
-				desc = ""
-			desc += self.queryv1("DISPLAY") or ""
+				tech = self.queryv1("TECHNOLOGY")
+				if tech:
+					if (desc and desc != "Not specified" and
+						desc != "CCSS for " + tech):
+						display = desc
+					else:
+						display = self.queryv1("DISPLAY")
+					if display:
+						tech += " (%s)" % display
+				desc = tech
 		if not desc and self.filename:
 			desc = os.path.splitext(os.path.basename(self.filename))[0]
 		return desc
@@ -374,7 +399,7 @@ class CGATS(dict):
 	def __setattr__(self, name, value):
 		if name == 'modified':
 			self.setmodified(value)
-		elif name in ('datetime', 'filename', 'file_identifier', 'key', 
+		elif name in ('datetime', 'filename', 'fileName', 'file_identifier', 'key', 
 					  'mtime', 'normalize_fields', 'parent', 'root', 'type', 
 					  'vmaxlen'):
 			object.__setattr__(self, name, value)
@@ -433,7 +458,7 @@ class CGATS(dict):
 								result.append('%s "%s"' % (key, value))
 				elif key not in ('DATA_FORMAT', 'KEYWORDS'):
 					if (value.type == 'SECTION' and result[-1:] and 
-						result[-1:][0][-1] != '\n'):
+						result[-1:][0] != ''):
 						result.append('')
 					result.append(str(value))
 			if self.type == 'SECTION':
@@ -567,6 +592,34 @@ class CGATS(dict):
 		valueslist.sort(sort_RGB_gray_to_top)
 		return data.set_RGB_XYZ_values(valueslist)
 	
+	def sort_RGB_to_top(self, r=0, g=0, b=0):
+		"""
+		Sort quantities of R, G or B (or combinations) to top.
+		
+		Example: sort_RGB_to_top(True, 0, 0) - sort red values to top
+		Example: sort_RGB_to_top(0, True, True) - sort cyan values to top
+		
+		"""
+		data, valueslist = self.get_RGB_XYZ_values()
+		if not valueslist:
+			return False
+		if r and g and b:
+			fn = sort_RGB_gray_to_top
+		elif r and g:
+			fn = sort_RGB_to_top_factory(0, 1, 2, 0)
+		elif r and b:
+			fn = sort_RGB_to_top_factory(0, 2, 1, 0)
+		elif g and b:
+			fn = sort_RGB_to_top_factory(1, 2, 0, 1)
+		elif r:
+			fn = sort_RGB_to_top_factory(1, 2, 1, 0)
+		elif g:
+			fn = sort_RGB_to_top_factory(0, 2, 0, 1)
+		elif b:
+			fn = sort_RGB_to_top_factory(0, 1, 0, 2)
+		valueslist.sort(fn)
+		return data.set_RGB_XYZ_values(valueslist)
+	
 	def sort_RGB_white_to_top(self):
 		data, valueslist = self.get_RGB_XYZ_values()
 		if not valueslist:
@@ -1590,10 +1643,18 @@ Transform {
 	
 	pop = remove
 	
-	def write(self, filename=None):
-		if not filename:
-			filename = self.filename
-		txt = open(filename, "w")
-		txt.write(str(self))
-		txt.close()
+	def write(self, stream_or_filename=None):
+		"""
+		Write CGATS text to stream.
+		
+		"""
+		if not stream_or_filename:
+			stream_or_filename = self.filename
+		if isinstance(stream_or_filename, basestring):
+			stream = open(stream_or_filename, "w")
+		else:
+			stream = stream_or_filename
+		stream.write(str(self))
+		if isinstance(stream_or_filename, basestring):
+			stream.close()
 	
