@@ -17,7 +17,7 @@ from time import gmtime, strftime, timezone
 if sys.platform == "win32":
 	import _winreg
 
-from argyll_names import viewconds, intents, video_encodings
+from argyll_names import observers, viewconds, intents, video_encodings
 from defaultpaths import appdata, commonappdata
 if sys.platform == "win32":
 	from defaultpaths import commonprogramfiles
@@ -34,7 +34,7 @@ from options import ascii, debug, verbose
 from safe_print import enc, fs_enc, original_codepage
 from util_io import StringIOu as StringIO
 from util_os import expanduseru, expandvarsu, getenvu, is_superuser, listdir_re
-from util_str import create_replace_function, safe_unicode
+from util_str import create_replace_function, safe_unicode, strtr
 import encodedstdio
 
 # Runtime configuration
@@ -67,6 +67,13 @@ else:
 							else os.path.abspath(unicode(__file__, fs_enc)))
 
 data_dirs = [pydir]
+extra_data_dirs = []
+# Search directories on PATH for data directories so Argyll reference files
+# can be found automatically if Argyll directory not explicitly configured
+for dir_ in getenvu("PATH", "").split(os.pathsep):
+	dir_parent = os.path.dirname(dir_)
+	if os.path.isdir(os.path.join(dir_parent, "ref")):
+		extra_data_dirs.append(dir_parent)
 
 if sys.platform == "win32":
 	if pydir.lower().startswith(exedir.lower()) and pydir != exedir:
@@ -111,10 +118,10 @@ else:
 		if not datahome_default in data_dirs:
 			data_dirs.append(datahome_default)
 		data_dirs.extend(os.path.join(dir_, appname) for dir_ in xdg_data_dirs)
-		data_dirs.extend(os.path.join(dir_, "argyllcms") for dir_ in
-						 xdg_data_dirs)
-		data_dirs.extend(os.path.join(dir_, "color", "argyll") for dir_ in
-						 xdg_data_dirs)
+		extra_data_dirs.extend(os.path.join(dir_, "argyllcms") for dir_ in
+							   xdg_data_dirs)
+		extra_data_dirs.extend(os.path.join(dir_, "color", "argyll") for dir_ in
+							   xdg_data_dirs)
 	exe_ext = ""
 	profile_ext = ".icc"
 
@@ -130,7 +137,7 @@ resfiles = [
 	"test.cal",
 	"ref/ClayRGB1998.gam",
 	"ref/sRGB.gam",
-	"ref/verify.ti1",
+	"ref/verify_extended.ti1",
 	"xrc/3dlut.xrc",
 	"xrc/extra.xrc",
 	"xrc/gamap.xrc",
@@ -142,17 +149,46 @@ resfiles = [
 
 bitmaps = {}
 
-uncalibratable_displays = ("Untethered", )
+uncalibratable_displays = ("Untethered$", )
 
-patterngenerators = ("madVR", "Resolve")
+patterngenerators = ("madVR$", "Resolve$", "Chromecast ")
 
-non_argyll_displays = uncalibratable_displays + ("Resolve", )
+non_argyll_displays = uncalibratable_displays + ("Resolve$", )
 
-untethered_displays = non_argyll_displays + ("Web", )
+untethered_displays = non_argyll_displays + ("Web$", "Chromecast ")
 
-virtual_displays = untethered_displays + ("madVR", )
+virtual_displays = untethered_displays + ("madVR$", )
 
-def getbitmap(name):
+
+def is_special_display(display_no=None, tests=virtual_displays):
+	display_name = get_display_name(display_no)
+	for test in tests:
+		if re.match(test, display_name):
+			return True
+	return False
+
+
+def is_uncalibratable_display(display_no=None):
+	return is_special_display(display_no, uncalibratable_displays)
+
+
+def is_patterngenerator(display_no=None):
+	return is_special_display(display_no, patterngenerators)
+
+
+def is_non_argyll_display(display_no=None):
+	return is_special_display(display_no, non_argyll_displays)
+
+
+def is_untethered_display(display_no=None):
+	return is_special_display(display_no, untethered_displays)
+
+
+def is_virtual_display(display_no=None):
+	return is_special_display(display_no, virtual_displays)
+
+
+def getbitmap(name, display_missing_icon=True):
 	"""
 	Create (if necessary) and return a named bitmap.
 	
@@ -168,16 +204,16 @@ def getbitmap(name):
 	from wxaddons import wx
 	if not name in bitmaps:
 		parts = name.split("/")
+		w = 16
+		h = 16
+		if len(parts) > 1:
+			size = parts[-2].split("x")
+			if len(size) == 2:
+				try:
+					w, h = map(int, size)
+				except ValueError:
+					pass
 		if parts[-1] == "empty":
-			w = 16
-			h = 16
-			if len(parts) > 1:
-				size = parts[-2].split("x")
-				if len(size) == 2:
-					try:
-						w, h = map(int, size)
-					except ValueError:
-						pass
 			bitmaps[name] = wx.EmptyBitmap(w, h, depth=-1)
 			dc = wx.MemoryDC()
 			dc.SelectObject(bitmaps[name])
@@ -194,17 +230,18 @@ def getbitmap(name):
 			if path:
 				bitmaps[name] = wx.Bitmap(path)
 			else:
-				w = 32
-				h = 32
-				if len(parts) > 1:
-					size = parts[-2].split("x")
-					if len(size) == 2:
-						try:
-							w, h = map(int, size)
-						except ValueError:
-							pass
-				bitmaps[name] = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE, 
-														 size=(w, h))
+				img = wx.EmptyImage(w, h)
+				img.SetMaskColour(0, 0, 0)
+				img.InitAlpha()
+				bmp = img.ConvertToBitmap()
+				bitmaps[name] = bmp
+				dc = wx.MemoryDC()
+				dc.SelectObject(bitmaps[name])
+				if display_missing_icon:
+					bmp = wx.ArtProvider.GetBitmap(wx.ART_MISSING_IMAGE,
+												   size=(w, h))
+					dc.DrawBitmap(bmp, 0, 0, True)
+				dc.SelectObject(wx.NullBitmap)
 	return bitmaps[name]
 
 
@@ -237,19 +274,33 @@ def get_display_name(n=None, include_geometry=False):
 	""" Return name of currently configured display """
 	if n is None:
 		n = getcfg("display.number") - 1
-	displays = getcfg("displays").split(os.pathsep)
+	displays = getcfg("displays")
 	if n >= 0 and n < len(displays):
 		if include_geometry:
 			return displays[n]
 		else:
-			return displays[n].split("@")[0].strip()
+			return split_display_name(displays[n])
 	return ""
 
 
+def split_display_name(display):
+	"""
+	Split and return name part of display
+	
+	E.g.
+	'LCD2690WUXi @ 0, 0, 1920x1200' -> 'LCD2690WUXi'
+	'madVR' -> 'madVR'
+	
+	"""
+	if "@" in display and not display.startswith("Chromecast "):
+		display = "@".join(display.split("@")[:-1])
+	return display.strip()
+
+
 def get_argyll_display_number(geometry):
 	""" Translate from wx display geometry to Argyll display index """
 	geometry = "%i, %i, %ix%i" % tuple(geometry)
-	for i, display in enumerate(getcfg("displays").split(os.pathsep)):
+	for i, display in enumerate(getcfg("displays")):
 		if display.find("@ " + geometry) > -1:
 			if debug:
 				from log import safe_print
@@ -260,14 +311,14 @@ def get_argyll_display_number(geometry):
 
 def get_display_number(display_no):
 	""" Translate from Argyll display index to wx display index """
+	if is_virtual_display(display_no):
+		return 0
 	from wxaddons import wx
 	try:
-		display = getcfg("displays").split(os.pathsep)[display_no]
+		display = getcfg("displays")[display_no]
 	except IndexError:
 		return 0
 	else:
-		if display.split("@")[0].strip() in virtual_displays:
-			return 0
 		for i in xrange(wx.Display.GetCount()):
 			geometry = "%i, %i, %ix%i" % tuple(wx.Display(i).Geometry)
 			if display.find("@ " + geometry) > -1:
@@ -283,7 +334,7 @@ def get_display_rects():
 	""" Return the Argyll enumerated display coordinates and sizes """
 	from wxaddons import wx
 	display_rects = []
-	for i, display in enumerate(getcfg("displays").split(os.pathsep)):
+	for i, display in enumerate(getcfg("displays")):
 		match = re.search("@ (-?\d+), (-?\d+), (\d+)x(\d+)", display)
 		if match:
 			display_rects.append(wx.Rect(*[int(item) for item in match.groups()]))
@@ -302,12 +353,24 @@ def get_icon_bundle(sizes, name):
 def get_instrument_name():
 	""" Return name of currently configured instrument """
 	n = getcfg("comport.number") - 1
-	instrument_names = getcfg("instruments").split(os.pathsep)
+	instrument_names = getcfg("instruments")
 	if n >= 0 and n < len(instrument_names):
 		return instrument_names[n]
 	return ""
 
 
+def get_measureframe_dimensions(dimensions_measureframe=None, percent=10):
+	""" return measurement area size adjusted for percentage of screen area """
+	if not dimensions_measureframe:
+		dimensions_measureframe = getcfg("dimensions.measureframe")
+	dimensions_measureframe = [float(n) for n in
+							   dimensions_measureframe.split(",")]
+	dimensions_measureframe[2] *= defaults["size.measureframe"]
+	dimensions_measureframe[2] /= get_display_rects()[0][2]
+	dimensions_measureframe[2] *= percent
+	return ",".join([str(min(n, 50)) for n in dimensions_measureframe])
+
+
 def geticon(size, name):
 	""" Convenience function for getbitmap('theme/icons/<size>/<name>'). """
 	return getbitmap("theme/icons/%(size)sx%(size)s/%(name)s" % 
@@ -330,6 +393,7 @@ def get_data_path(relpath, rex=None):
 	argyll_dir = getcfg("argyll.dir")
 	if argyll_dir and os.path.isdir(os.path.join(argyll_dir, "..", "ref")):
 		dirs.append(os.path.dirname(argyll_dir))
+	dirs.extend(extra_data_dirs)
 	intersection = []
 	paths = []
 	for dir_ in dirs:
@@ -362,11 +426,7 @@ def get_data_path(relpath, rex=None):
 			else:
 				return curpath
 	if paths:
-		paths = [os.pathsep.join(reversed(os.path.split(path))) for path in
-				 paths]
-		paths.sort()
-		paths = [os.path.join(*reversed(path.split(os.pathsep))) for path in
-				 paths]
+		paths.sort(key=lambda path: os.path.basename(path).lower())
 	return None if len(paths) == 0 else paths
 
 
@@ -379,7 +439,7 @@ def runtimeconfig(pyfile):
 	
 	"""
 	from log import setup_logging
-	setup_logging(logdir)
+	setup_logging(logdir, pyname)
 	if debug or verbose >= 1:
 		from log import safe_print
 	if debug:
@@ -398,8 +458,8 @@ def runtimeconfig(pyfile):
 			safe_print("[D] dirname(os.path.abspath(sys.argv[0])):", 
 					   os.path.dirname(os.path.abspath(sys.argv[0])))
 			safe_print("[D] pydir parent:", pydir_parent)
-		if os.path.dirname(os.path.abspath(sys.argv[0])) == pydir_parent and \
-		   pydir_parent not in data_dirs:
+		if (os.path.dirname(os.path.abspath(sys.argv[0])).decode(fs_enc) ==
+			pydir_parent and pydir_parent not in data_dirs):
 			# Add the parent directory of the package directory to our list
 			# of data directories if it is the directory containing the 
 			# currently run script (e.g. when running from source)
@@ -429,12 +489,12 @@ def runtimeconfig(pyfile):
 	defaultmmode = defaults["measurement_mode"]
 	defaultptype = defaults["profile.type"]
 	defaultchart = testchart_defaults.get(defaultptype, 
-										  testchart_defaults["s"])[None]
+										  testchart_defaults["l"])[None]
 	defaults["testchart.file"] = get_data_path(os.path.join("ti1", 
 															defaultchart)) or ""
 	defaults["testchart.file.backup"] = defaults["testchart.file"]
 	defaults["measurement_report.chart"] = get_data_path(os.path.join("ref", 
-															"verify.ti1")) or ""
+															"verify_extended.ti1")) or ""
 	return runtype
 
 # User settings
@@ -445,6 +505,7 @@ cfg.optionxform = str
 valid_ranges = {
 	"3dlut.trc_gamma": [0.000001, 10],
 	"3dlut.trc_output_offset": [0.0, 1.0],
+	"app.port": [1, 65535],
 	"gamma": [0.000001, 10],
 	"trc": [0.000001, 10],
 	"calibration.ambient_viewcond_adjust.lux": [0, sys.maxint],
@@ -453,11 +514,14 @@ valid_ranges = {
 	"calibration.black_point_correction": [0, 1],
 	"calibration.black_point_rate": [0.05, 20],
 	"calibration.luminance": [0.000001, 100000],
+	"iccgamut.surface_detail": [1.0, 50.0],
 	"measurement_report.trc_gamma": [0.01, 10],
 	"measurement_report.trc_output_offset": [0.0, 1.0],
 	"measure.display_settle_time_mult": [0.000001, 10000.0],
 	"measure.min_display_update_delay_ms": [20, 60000],
 	"patterngenerator.apl": [0.0, 1.0],
+	"patterngenerator.resolve.port": [1, 65535],
+	"synthprofile.trc_gamma": [0.01, 10],
 	"synthprofile.trc_output_offset": [0.0, 1.0],
 	"tc_export_repeat_patch_max": [1, 1000],
 	"tc_export_repeat_patch_min": [1, 1000],
@@ -470,13 +534,18 @@ valid_values = {
 	"3d.format": ["HTML", "VRML", "X3D"],
 	"3dlut.bitdepth.input": [8, 10, 12, 14, 16],
 	"3dlut.bitdepth.output": [8, 10, 12, 14, 16],
-	"3dlut.encoding.input": video_encodings,
-	"3dlut.encoding.output": video_encodings,
+	"3dlut.encoding.input": list(video_encodings),
+	# collink: xvYCC output encoding is not supported
+	"3dlut.encoding.output": filter(lambda v: v not in ("T", "x", "X"),
+									video_encodings),
 	"3dlut.format": ["3dl", "cube", "eeColor", "madVR", "mga", "spi3d"],
 	"3dlut.rendering_intent": intents,
 	"3dlut.size": [17, 24, 32, 33, 64, 65],
 	"3dlut.trc_gamma_type": ["b", "B"],
 	"calibration.quality": ["v", "l", "m", "h", "u"],
+	"colorimeter_correction.observer": observers,
+	"colorimeter_correction.observer.reference": observers,
+	"colorimeter_correction.type": ["matrix", "spectral"],
 	# Measurement modes as supported by Argyll -y parameter
 	# 'l' = 'n' (non-refresh-type display, e.g. LCD)
 	# 'c' = 'r' (refresh-type display, e.g. CRT)
@@ -491,8 +560,8 @@ valid_values = {
 	"gamap_saturation_intent": intents,
 	"gamap_src_viewcond": viewconds,
 	"gamap_out_viewcond": ["mt", "mb", "md", "jm", "jd"],
-	"measure.uniformity.show_detail": [0, 1],
 	"measurement_report.trc_gamma_type": ["b", "B"],
+	"observer": observers,
 	"patterngenerator.resolve.use_video_levels": [0, 1],
 	"profile.black_point_compensation": [0, 1],
 	"profile.install_scope": ["l", "u"],
@@ -507,6 +576,7 @@ valid_values = {
 	"tc_vrml_cie_colorspace": ["DIN99", "DIN99b", "DIN99c", "DIN99d", "LCH(ab)",
 							   "LCH(uv)", "Lab", "Luv", "Lu'v'", "xyY"],
 	"tc_vrml_device_colorspace": ["HSI", "HSL", "HSV", "RGB"],
+	"testchart.auto_optimize": [0, 3, 4, 5, 6, 7, 8, 9, 10, 11, 12, 13, 14, 15, 16, 17, 18],
 	"trc": ["240", "709", "l", "s", ""],
 	"trc.type": ["g", "G"],
 	"whitepoint.colortemp.locus": ["t", "T"]
@@ -514,26 +584,34 @@ valid_values = {
 
 defaults = {
 	"3d.format": "HTML",
+	"3dlut.apply_black_offset": 0,
 	"3dlut.apply_trc": 1,
 	"3dlut.bitdepth.input": 10,
 	"3dlut.bitdepth.output": 10,
+	"3dlut.create": 0,
 	"3dlut.trc_gamma": 2.4,
+	"3dlut.trc_gamma.backup": 2.4,
 	"3dlut.trc_gamma_type": "B",
 	"3dlut.trc_output_offset": 0.0,
 	"3dlut.encoding.input": "n",
 	"3dlut.encoding.input.backup": "n",
 	"3dlut.encoding.output": "n",
 	"3dlut.encoding.output.backup": "n",
-	"3dlut.format": "3dl",
+	"3dlut.format": "cube",
+	"3dlut.gamap.use_b2a": 0,
 	"3dlut.input.profile": "",
 	"3dlut.abstract.profile": "",
 	"3dlut.madVR.enable": 1,
 	"3dlut.output.profile": "",
 	"3dlut.output.profile.apply_cal": 1,
-	"3dlut.rendering_intent": "r",
+	"3dlut.rendering_intent": "aw",
 	"3dlut.use_abstract_profile": 0,
-	"3dlut.size": 17,
+	"3dlut.size": 65,
+	"3dlut.tab.enable": 0,
+	"3dlut.tab.enable.backup": 0,
 	"allow_skip_sensor_cal": 0,
+	"app.allow_network_clients": 0,
+	"app.port": 15411,
 	"argyll.debug": 0,
 	"argyll.dir": None,
 	"argyll.version": "0.0.0",
@@ -549,10 +627,11 @@ defaults = {
 	"calibration.black_point_correction": 0.0,
 	"calibration.black_point_correction.auto": 0,
 	"calibration.black_point_correction_choice.show": 1,
+	"calibration.black_point_hack": 0,
 	"calibration.black_point_rate": 4.0,
 	"calibration.black_point_rate.enabled": 0,
 	"calibration.continue_next": 0,
-	"calibration.file": None,
+	"calibration.file": "presets/default.icc",
 	"calibration.file.previous": None,
 	"calibration.interactive_display_adjustment": 1,
 	"calibration.interactive_display_adjustment.backup": 1,
@@ -562,9 +641,20 @@ defaults = {
 	"calibration.update": 0,
 	"calibration.use_video_lut": 1,
 	"calibration.use_video_lut.backup": 1,
+	"colorimeter_correction.instrument": None,
+	"colorimeter_correction.instrument.reference": None,
+	"colorimeter_correction.measurement_mode": "l",
+	"colorimeter_correction.measurement_mode.reference.adaptive": 1,
+	"colorimeter_correction.measurement_mode.reference.highres": 0,
+	"colorimeter_correction.measurement_mode.reference.projector": 0,
+	"colorimeter_correction.measurement_mode.reference": "l",
+	"colorimeter_correction.observer": "1931_2",
+	"colorimeter_correction.observer.reference": "1931_2",
 	"colorimeter_correction.testchart": "ccxx.ti1",
 	"colorimeter_correction_matrix_file": "AUTO:",
+	"colorimeter_correction.type": "matrix",
 	"comport.number": 1,
+	"comport.number.backup": 1,
 	# Note: worker.Worker.enumerate_displays_and_ports() overwrites copyright
 	"copyright": "No copyright. Created with %s %s and Argyll CMS" % (appname, 
 																	  version),
@@ -573,6 +663,7 @@ defaults = {
 	"display.number": 1,
 	"display_lut.link": 1,
 	"display_lut.number": 1,
+	"display.technology": "LCD",
 	"displays": "",
 	"dry_run": 0,
 	"enumerate_ports.auto": 0,
@@ -589,21 +680,27 @@ defaults = {
 	"gamap_saturation_intent": "s",
 	"gamap_default_intent": "p",
 	"gamma": 2.2,
+	"iccgamut.surface_detail": 6.0,
 	"instruments": "",
-	"log.autoshow": int(not(hasattr(sys.stdout, "isatty") and 
-							sys.stdout.isatty())),
+	"last_launch": "99",
+	"log.autoshow": 0,
 	"log.show": 0,
 	"lang": "en",
 	# The last_[...]_path defaults are set in localization.py
 	"lut_viewer.show": 0,
 	"lut_viewer.show_actual_lut": 0,
+	"madtpg.host": "localhost",
+	"madtpg.native": 1,
+	"madtpg.port": 60562,
 	"measurement_mode": "l",
 	"measurement_mode.adaptive": 1,
 	"measurement_mode.backup": "l",
 	"measurement_mode.highres": 0,
 	"measurement_mode.projector": 0,
+	"measurement_report.apply_black_offset": 0,
 	"measurement_report.apply_trc": 0,
 	"measurement_report.trc_gamma": 2.4,
+	"measurement_report.trc_gamma.backup": 2.4,
 	"measurement_report.trc_gamma_type": "B",
 	"measurement_report.trc_output_offset": 0.0,
 	"measurement_report.chart": "",
@@ -616,7 +713,6 @@ defaults = {
 	"measurement_report.use_devlink_profile": 0,
 	"measurement_report.use_simulation_profile": 0,
 	"measurement_report.use_simulation_profile_as_output": 0,
-	"measurement.continuous": 0,
 	"measurement.name.expanded": u"",
 	"measurement.play_sound": 1,
 	"measurement.save_path": expanduseru("~"),
@@ -630,9 +726,10 @@ defaults = {
 	"measure.override_display_settle_time_mult.backup": 0,
 	"measure.override_min_display_update_delay_ms": 0,
 	"measure.override_min_display_update_delay_ms.backup": 0,
-	"measure.uniformity.show_detail": 0,
+	"observer": "1931_2",
+	"observer.backup": "1931_2",
 	"patterngenerator.apl": .22,
-	"patterngenerator.resolve.bits": 10,
+	"patterngenerator.resolve": "CM",
 	"patterngenerator.resolve.port": 20002,
 	"patterngenerator.resolve.use_video_levels": 0,
 	"position.x": 50,
@@ -649,12 +746,13 @@ defaults = {
 	"position.progress.y": 50,
 	"position.reportframe.x": 50,
 	"position.reportframe.y": 50,
+	"position.scripting.x": 50,
+	"position.scripting.y": 50,
 	"position.synthiccframe.x": 50,
 	"position.synthiccframe.y": 50,
 	"position.tcgen.x": 50,
 	"position.tcgen.y": 50,
-	"profile.black_point_compensation": 1,
-	"profile.black_point_compensation.backup": 1,
+	"profile.black_point_compensation": 0,
 	"profile.create_gamut_views": 1,
 	"profile.install_scope": "l" if (sys.platform != "win32" and 
 									 os.geteuid() == 0) # or 
@@ -665,7 +763,8 @@ defaults = {
 	"profile.load_on_login": 1,
 	"profile.name": u" ".join([
 		u"%dns",
-		u"%Y-%m-%d",
+		u"%out",
+		u"%Y-%m-%d %H-%M",
 		u"%cb",
 		u"%wp",
 		u"%cB",
@@ -682,14 +781,15 @@ defaults = {
 	"profile.b2a.hires.size": 33,
 	"profile.b2a.hires.smooth": 1,
 	"profile.save_path": storage, # directory
-	"profile.type": "S",
+	"profile.type": "X",
 	"profile.update": 0,
 	"profile_loader.error.show_msg": 1,
 	"profile_loader.verify_calibration": 0,
 	"recent_cals": "",
 	"report.pack_js": 1,
 	"settings.changed": 0,
-	"show_advanced_calibration_options": 0,
+	"show_advanced_options": 0,
+	"show_donation_message": 1,
 	"size.info.w": 512,
 	"size.info.h": 384,
 	"size.lut3dframe.w": 512,
@@ -702,10 +802,14 @@ defaults = {
 	"size.lut_viewer.h": 552,
 	"size.reportframe.w": 512,
 	"size.reportframe.h": 256,
+	"size.scripting.w": 512,
+	"size.scripting.h": 384,
 	"size.synthiccframe.w": 512,
 	"size.synthiccframe.h": 384,
 	"skip_legacy_serial_ports": 1,
 	"skip_scripts": 1,
+	"splash.zoom": 0,
+	"startup_sound.enable": 1,
 	"sudo.preserve_environment": 1,
 	"synthprofile.black_luminance": 0.0,
 	"synthprofile.luminance": 120.0,
@@ -747,6 +851,8 @@ defaults = {
 	"tc_vrml_use_D50": 0,
 	"tc_white_patches": 4,
 	"tc.show": 0,
+	"testchart.auto_optimize": 5,
+	"testchart.auto_optimize.fix_zero_blackpoint": 0,
 	"testchart.reference": "",
 	"ti3.check_sanity.auto": 0,
 	"trc": 2.2,
@@ -754,12 +860,14 @@ defaults = {
 	"trc.should_use_viewcond_adjust.show_msg": 1,
 	"trc.type": "g",
 	"trc.type.backup": "g",
+	"uniformity.measure.continuous": 0,
 	"untethered.measure.auto": 1,
 	"untethered.measure.manual.delay": 0.75,
 	"untethered.max_delta.chroma": 0.5,
 	"untethered.min_delta": 1.5,
 	"untethered.min_delta.lightness": 1.0,
 	"update_check": 1,
+	"use_fancy_progress": 1,
 	"use_separate_lut_access": 0,
 	"vrml.compress": 1,
 	"webserver.portnumber": 8080,
@@ -778,9 +886,9 @@ if lcode:
 	defaults["lang"] = lcode.split("_")[0].lower()
 
 testchart_defaults = {
-	"s": {None: "d3-e4-s0-g9-m3-b3-f0-crossover.ti1"},  # shaper + matrix
+	"s": {None: "d3-e4-s0-g25-m3-b3-f0-crossover.ti1"},  # shaper + matrix
 	"l": {None: "d3-e4-s17-g49-m5-b5-f0.ti1"},  # lut
-	"g": {None: "d3-e4-s3-g3-m0-f0.ti1"}  # gamma + matrix
+	"g": {None: "d3-e4-s0-g25-m3-b3-f0-crossover.ti1"}  # gamma + matrix
 }
 
 def _init_testcharts():
@@ -793,7 +901,7 @@ def _init_testcharts():
 		testchart_defaults[key] = testchart_defaults["l"]
 
 
-def getcfg(name, fallback=True):
+def getcfg(name, fallback=True, raw=False):
 	"""
 	Get and return an option value from the configuration.
 	
@@ -801,12 +909,9 @@ def getcfg(name, fallback=True):
 	return its default value.
 	
 	"""
-	if (name in ("profile.save_path", "profile.name.expanded") and
-		is_ccxx_testchart()):
-		name = {"profile.save_path": "measurement.save_path",
-				"profile.name.expanded": "measurement.name.expanded"}[name]
-		setcfg("measurement.name.expanded", "%s & %s" % (get_instrument_name(),
-													     get_display_name()))
+	if name == "profile.name.expanded" and is_ccxx_testchart():
+		name = "measurement.name.expanded"
+	value = None
 	hasdef = name in defaults
 	if hasdef:
 		defval = defaults[name]
@@ -818,7 +923,9 @@ def getcfg(name, fallback=True):
 			pass
 		else:
 			# Check for invalid types and return default if wrong type
-			if (name != "trc" or value not in valid_values["trc"]) and \
+			if raw:
+				pass
+			elif (name != "trc" or value not in valid_values["trc"]) and \
 			   hasdef and deftype in (Decimal, int, float):
 				try:
 					value = deftype(value)
@@ -848,32 +955,10 @@ def getcfg(name, fallback=True):
 				value = "h"
 			elif name == "trc.type" and getcfg("trc") in valid_values["trc"]:
 				value = "g"
-			elif (value and name.endswith("file") and
-				  name != "colorimeter_correction_matrix_file" and
-				  not os.path.exists(value)):
-				  # colorimeter_correction_matrix_file is special because it's
-				  # not (only) a path
-				if debug:
-					print "%s does not exist: %s" % (name, value),
-				if value.split(os.path.sep)[-3:-2] == [appname] and (
-				   value.split(os.path.sep)[-2:-1] == ["presets"] or 
-				   value.split(os.path.sep)[-2:-1] == ["ref"] or 
-				   value.split(os.path.sep)[-2:-1] == ["ti1"]):
-					value = os.path.join(*value.split(os.path.sep)[-2:])
-					value = get_data_path(value)
-				if not value and hasdef:
-					value = defval
-				if debug:
-					print "- falling back to", value
 			elif name in valid_values and value not in valid_values[name]:
 				if debug:
 					print "Invalid config value for %s: %s" % (name, value),
-				if hasdef:
-					value = defval
-				else:
-					value = None
-				if debug:
-					print "- falling back to", value
+				value = None
 			elif name == "copyright":
 				# Make sure dispcalGUI and Argyll version are up-to-date
 				pattern = re.compile("(%s(?:\s*v(?:ersion|\.)?)?\s*)\d+(?:\.\d+)*" %
@@ -894,13 +979,45 @@ def getcfg(name, fallback=True):
 				# Worker.add_measurement_features(). That way we can have
 				# compatibility with old and current Argyll CMS
 				value = {"n": "l", "r": "c"}.get(value, value)
-			return value
-	if hasdef and fallback:
-		value = defval
-	else:
-		if debug and not hasdef: 
-			print "Warning - unknown option:", name
-		value = None
+	if value is None:
+		if hasdef and fallback:
+			value = defval
+			if debug:
+				print "- falling back to", value
+		else:
+			if debug and not hasdef: 
+				print "Warning - unknown option:", name
+	if raw:
+		return value
+	if (value and isinstance(value, basestring) and
+		name.endswith("file") and
+		name != "colorimeter_correction_matrix_file" and
+		(name != "testchart.file" or value != "auto") and
+		(not os.path.isabs(value) or not os.path.exists(value))):
+		# colorimeter_correction_matrix_file is special because it's
+		# not (only) a path
+		if debug:
+			print "%s does not exist: %s" % (name, value),
+		# Normalize path (important, this turns altsep into sep under
+		# Windows)
+		value = os.path.normpath(value)
+		# Check if this is a relative path covered by data_dirs
+		if (value.split(os.path.sep)[-3:-2] == [appname] or
+			not os.path.isabs(value)) and (
+		   value.split(os.path.sep)[-2:-1] == ["presets"] or 
+		   value.split(os.path.sep)[-2:-1] == ["ref"] or 
+		   value.split(os.path.sep)[-2:-1] == ["ti1"]):
+			value = os.path.join(*value.split(os.path.sep)[-2:])
+			value = get_data_path(value)
+		elif hasdef:
+			value = None
+		if not value and hasdef:
+			value = defval
+		if debug:
+			print "- falling back to", value
+	elif name in ("displays", "instruments"):
+		value = [strtr(v, [("%" + hex(ord(os.pathsep))[2:].upper(), os.pathsep),
+						   ("%25", "%")]) for v in value.split(os.pathsep)]
 	return value
 
 
@@ -928,7 +1045,7 @@ def get_ccxx_testchart():
 
 def get_current_profile(include_display_profile=False):
 	""" Get the currently selected profile (if any) """
-	path = getcfg("calibration.file")
+	path = getcfg("calibration.file", False)
 	if path:
 		import ICCProfile as ICCP
 		try:
@@ -943,7 +1060,7 @@ def get_current_profile(include_display_profile=False):
 def get_display_profile(display_no=None):
 	if display_no is None:
 		display_no = max(getcfg("display.number") - 1, 0)
-	if get_display_name(display_no) in virtual_displays:
+	if is_virtual_display(display_no):
 		return None
 	import ICCProfile as ICCP
 	try:
@@ -1110,7 +1227,7 @@ def is_ccxx_testchart(testchart=None):
 
 
 def is_profile(filename=None, include_display_profile=False):
-	filename = filename or getcfg("calibration.file")
+	filename = filename or getcfg("calibration.file", False)
 	if filename:
 		if os.path.exists(filename):
 			import ICCProfile as ICCP
@@ -1157,7 +1274,7 @@ def makecfgdir(which="user", worker=None):
 	return True
 
 
-def initcfg():
+def initcfg(module=None):
 	"""
 	Initialize the configuration.
 	
@@ -1165,6 +1282,10 @@ def initcfg():
 	settings directory if nonexistent.
 	
 	"""
+	if module:
+		cfgbasename = "%s-%s" % (appname, module)
+	else:
+		cfgbasename = appname
 	# read pre-v0.2.2b configuration if present
 	if sys.platform == "darwin":
 		oldcfg = os.path.join(expanduseru("~"), "Library", "Preferences", 
@@ -1173,13 +1294,13 @@ def initcfg():
 		oldcfg = os.path.join(expanduseru("~"), "." + appname)
 	makecfgdir()
 	if os.path.exists(confighome) and \
-	   not os.path.exists(os.path.join(confighome, appname + ".ini")):
+	   not os.path.exists(os.path.join(confighome, cfgbasename + ".ini")):
 		try:
 			if os.path.isfile(oldcfg):
 				oldcfg_file = open(oldcfg, "rb")
 				oldcfg_contents = oldcfg_file.read()
 				oldcfg_file.close()
-				cfg_file = open(os.path.join(confighome, appname + ".ini"), 
+				cfg_file = open(os.path.join(confighome, cfgbasename + ".ini"), 
 								"wb")
 				cfg_file.write("[Default]\n" + oldcfg_contents)
 				cfg_file.close()
@@ -1187,7 +1308,7 @@ def initcfg():
 				key = _winreg.OpenKey(_winreg.HKEY_CURRENT_USER, 
 									  "Software\\" + appname)
 				numsubkeys, numvalues, mtime = _winreg.QueryInfoKey(key)
-				cfg_file = open(os.path.join(confighome, appname + ".ini"), 
+				cfg_file = open(os.path.join(confighome, cfgbasename + ".ini"), 
 								"wb")
 				cfg_file.write("[Default]\n")
 				for i in range(numvalues):
@@ -1208,15 +1329,42 @@ def initcfg():
 		setcfg("gamap_src_viewcond", "mt")
 		setcfg("gamap_out_viewcond", "mt")
 	# Read cfg
+	cfgnames = []
+	if module != "3DLUT-maker":
+		cfgnames.append(appname)
+	if module:
+		cfgnames.append(cfgbasename)
+	else:
+		cfgnames.extend("%s-%s" % (appname, othermod) for othermod in
+						("synthprofile", "testchart-editor"))
+	cfgroots = [confighome]
+	if module == "apply-profiles":
+		cfgroots.append(config_sys)
+	cfgfiles = []
+	for cfgname in cfgnames:
+		for cfgroot in cfgroots:
+			cfgfile = os.path.join(cfgroot, cfgname + ".ini")
+			if os.path.isfile(cfgfile):
+				cfgfiles.append(cfgfile)
+				# Make user config take precedence
+				break
+	cfgfiles.sort(key=lambda cfgfile: os.stat(cfgfile).st_mtime)
 	try:
-		cfg.read([os.path.join(config_sys, appname + ".ini")])
-		cfg.read([os.path.join(confighome, appname + ".ini")])
-		# This won't raise an exception if the file does not exist, only if it
-		# can't be parsed
+		cfg.read(cfgfiles)
+		# This won't raise an exception if the file does not exist, only
+		# if it can't be parsed
 	except Exception, exception:
 		from log import safe_print
-		safe_print("Warning - could not parse configuration file '%s'" % 
-				   appname + ".ini")
+		safe_print("Warning - could not parse configuration files:\n%s" %
+				   "\n".join(cfgfiles))
+		# Fix Python 2.7 ConfigParser option values being lists instead of
+		# strings in case of a ParsingError. http://bugs.python.org/issue2414
+		all_sections = [ConfigParser.DEFAULTSECT]
+		all_sections.extend(cfg.sections())
+		for section in all_sections:
+			for name, val in cfg.items(section):
+				if isinstance(val, list):
+					cfg.set(section, name, "\n".join(val))
 
 
 def setcfg(name, value):
@@ -1224,10 +1372,15 @@ def setcfg(name, value):
 	if value is None:
 		cfg.remove_option(ConfigParser.DEFAULTSECT, name)
 	else:
+		if name in ("displays", "instruments") and isinstance(value, (list,
+																	  tuple)):
+			value = os.pathsep.join(strtr(v, [("%", "%25"),
+											  (os.pathsep,
+											   "%" + hex(ord(os.pathsep))[2:].upper())]) for v in value)
 		cfg.set(ConfigParser.DEFAULTSECT, name, unicode(value).encode("UTF-8"))
 
 
-def writecfg(which="user", worker=None):
+def writecfg(which="user", worker=None, module=None, options=()):
 	"""
 	Write configuration file.
 	
@@ -1235,16 +1388,28 @@ def writecfg(which="user", worker=None):
 	worker: worker instance if which == 'system'
 	
 	"""
+	if module:
+		cfgbasename = "%s-%s" % (appname, module)
+	else:
+		cfgbasename = appname
 	if which == "user":
 		# user config - stores everything and overrides system-wide config
-		cfgfilename = os.path.join(confighome, appname + ".ini")
+		cfgfilename = os.path.join(confighome, cfgbasename + ".ini")
 		try:
 			io = StringIO()
 			cfg.write(io)
 			io.seek(0)
 			lines = io.read().strip("\n").split("\n")
+			if options:
+				optionlines = []
+				for optionline in lines[1:]:
+					for option in options:
+						if optionline.startswith(option):
+							optionlines.append(optionline)
+			else:
+				optionlines = lines[1:]
 			# Sorting works as long as config has only one section
-			lines = [lines[0]] + sorted(lines[1:])
+			lines = lines[:1] + sorted(optionlines)
 			cfgfile = open(cfgfilename, "wb")
 			cfgfile.write("\n".join(lines))
 			cfgfile.close()
@@ -1255,8 +1420,8 @@ def writecfg(which="user", worker=None):
 			return False
 	else:
 		# system-wide config - only stores essentials ie. Argyll directory
-		cfgfilename1 = os.path.join(confighome, appname + ".local.ini")
-		cfgfilename2 = os.path.join(config_sys, appname + ".ini")
+		cfgfilename1 = os.path.join(confighome, cfgbasename + ".local.ini")
+		cfgfilename2 = os.path.join(config_sys, cfgbasename + ".ini")
 		if sys.platform == "win32":
 			cfgfilename = cfgfilename2
 		else:
