@@ -1,5 +1,6 @@
 # -*- coding: utf-8 -*-
 
+import inspect
 import sys
 import traceback
 
@@ -49,8 +50,9 @@ def handle_error(error, parent=None, silent=False):
 		error = error[1]
 	else:
 		tbstr = traceback.format_exc()
-	if (debug or not isinstance(error, EnvironmentError) or
-				 not getattr(error, "filename", None)):
+	if (tbstr.strip() != "None" and isinstance(error, Exception) and
+		(debug or not isinstance(error, EnvironmentError) or
+		 not getattr(error, "filename", None))):
 		# Print a traceback if in debug mode, for non environment errors, and
 		# for environment errors not related to files
 		safe_print(tbstr)
@@ -62,6 +64,11 @@ def handle_error(error, parent=None, silent=False):
 			app = wx.GetApp()
 			if app is None and parent is None:
 				app = wx.App(redirect=False)
+				# wxPython 3 bugfix: We also need a toplevel window
+				frame = wx.Frame(None)
+				parent = False
+			else:
+				frame = None
 			if parent is None:
 				parent = app.GetTopWindow()
 			if parent:
@@ -70,15 +77,36 @@ def handle_error(error, parent=None, silent=False):
 				except:
 					# If the parent is still being constructed, we can't use it
 					parent = None
+			if isinstance(error, Warning):
+				icon = wx.ICON_WARNING
+			elif isinstance(error, Exception):
+				icon = wx.ICON_ERROR
+			else:
+				icon = wx.ICON_INFORMATION
 			dlg = wx.MessageDialog(parent if parent not in (False, None) and 
 								   parent.IsShownOnScreen() else None, 
-								   safe_unicode(error), appname, wx.OK |
-																 wx.ICON_ERROR)
-			dlg.ShowModal()
-			dlg.Destroy()
+								   safe_unicode(error), appname, wx.OK | icon)
+			if frame:
+				# wxPython 3 bugfix: We need to use CallLater and MainLoop
+				wx.CallLater(1, dlg.ShowModal)
+				wx.CallLater(1, frame.Close)
+				app.MainLoop()
+			else:
+				dlg.ShowModal()
+				dlg.Destroy()
 		except Exception, exception:
 			safe_print("Warning: handle_error():", safe_unicode(exception))
 
 
+def print_callstack():
+	""" Print call stack """
+	stack = inspect.stack()
+	indent = ""
+	for frame, filename, linenum, funcname, line, exc in reversed(stack[1:]):
+		safe_print(indent, funcname, filename, linenum,
+				   repr("".join(line).strip()))
+		indent += " "
+
+
 class ResourceError(Exception):
 	pass
