@@ -29,6 +29,7 @@ from distutils.command.install import install
 from distutils.util import change_root, get_platform
 from fnmatch import fnmatch
 import codecs
+import ctypes.util
 import distutils.core
 import glob
 import os
@@ -40,11 +41,31 @@ import sys
 from time import strftime
 from types import StringType
 
+
+# Borrowed from setuptools
+def findall(dir=os.curdir):
+	"""Find all files under 'dir' and return the list of full filenames
+	(relative to 'dir').
+	"""
+	all_files = []
+	for base, dirs, files in os.walk(dir, followlinks=True):
+		if base == os.curdir or base.startswith(os.curdir + os.sep):
+			base = base[2:]
+		if base:
+			files = [os.path.join(base, f) for f in files]
+		all_files.extend(filter(os.path.isfile, files))
+	return all_files
+
+import distutils.filelist
+distutils.filelist.findall = findall    # Fix findall bug in distutils
+
+
 from defaultpaths import autostart, autostart_home
 from meta import (author, author_ascii, description, longdesc, domain, name, 
 				  py_maxversion, py_minversion, version, version_tuple, 
 				  wx_minversion)
 from util_os import relpath
+appname = name
 
 bits = platform.architecture()[0][:2]
 pypath = os.path.abspath(__file__)
@@ -65,7 +86,13 @@ config = {"data": ["tests/*.icc"],
 		  # numpy.lib.utils imports pydoc, which imports Tkinter, but 
 		  # numpy.lib.utils is not even used by dispcalGUI, so omit all 
 		  # Tk stuff
-		  "excludes": {"all": ["Tkconstants", "Tkinter", "setuptools", "tcl",
+		  # Use pyglet with OpenAL as audio backend. We only need
+		  # pyglet, pyglet.app and pyglet.media
+		  "excludes": {"all": ["Tkconstants", "Tkinter", "pygame",
+							   "pyglet.canvas", "pyglet.extlibs", "pyglet.font",
+							   "pyglet.gl", "pyglet.graphics", "pyglet.image",
+							   "pyglet.input", "pyglet.text",
+							   "pyglet.window", "pyo", "setuptools", "tcl",
 							   "test"],
 					   "darwin": [],
 					   "win32": ["win32com.client.genpy"]},
@@ -87,10 +114,15 @@ config = {"data": ["tests/*.icc"],
 								  "technology_strings.json",
 								  "test.cal",
 								  "theme/*.png",
+								  "theme/*.wav",
 								  "theme/icons/10x10/*.png",
 								  "theme/icons/16x16/*.png",
 								  "theme/icons/32x32/*.png",
 								  "theme/icons/72x72/*.png",
+								  "theme/jet_anim/*.png",
+								  "theme/patch_anim/*.png",
+								  "theme/splash_anim/*.png",
+								  "theme/shutter_anim/*.png",
 								  "ti1/*.ti1",
 								  "x3d-viewer/*.css",
 								  "x3d-viewer/*.html",
@@ -264,7 +296,8 @@ def get_data(tgt_dir, key, pkgname=None, subkey=None, excludes=None):
 def get_scripts(excludes=None):
 	# It is required that each script has an accompanying .desktop file
 	scripts = []
-	desktopfiles = glob.glob(os.path.join(pydir, "..", "misc", "*.desktop"))
+	desktopfiles = glob.glob(os.path.join(pydir, "..", "misc",
+							 appname + "*.desktop"))
 	def sortbyname(a, b):
 		a, b = [os.path.splitext(v)[0] for v in (a, b)]
 		if a > b:
@@ -275,9 +308,11 @@ def get_scripts(excludes=None):
 			return 0
 	desktopfiles.sort(sortbyname)
 	for desktopfile in desktopfiles:
+		if desktopfile.startswith("z-"):
+			continue
 		cfg = ConfigParser()
 		cfg.read(desktopfile)
-		script = cfg.get("Desktop Entry", "Exec")
+		script = cfg.get("Desktop Entry", "Exec").split()[0]
 		if not filter(lambda exclude: fnmatch(script, exclude), excludes or []):
 			scripts.append((script,
 							cfg.get("Desktop Entry", "Name").decode("UTF-8")))
@@ -361,14 +396,14 @@ def setup():
 	
 	if do_py2exe:
 		origIsSystemDLL = py2exe.build_exe.isSystemDLL
+		systemroot = os.getenv("SystemRoot").lower()
 		def isSystemDLL(pathname):
-			if os.path.basename(pathname).lower() in ("gdiplus.dll", 
-													  "mfc90.dll"):
+			if (os.path.basename(pathname).lower() in ("gdiplus.dll", 
+													   "mfc90.dll") or
+				os.path.basename(pathname).lower().startswith("python") or
+				os.path.basename(pathname).lower().startswith("pywintypes")):
 				return 0
-			if os.path.basename(pathname).lower() in ("powrprof.dll", ) or \
-			   os.path.basename(pathname).lower().startswith("api-ms-win-"):
-				return 1
-			return origIsSystemDLL(pathname)
+			return pathname.lower().startswith(systemroot + "\\")
 		py2exe.build_exe.isSystemDLL = isSystemDLL
 
 	if do_uninstall:
@@ -460,7 +495,10 @@ def setup():
 	if sys.platform == "win32" and not do_py2exe:
 		package_data[name].append("theme/icons/*.ico")
 	# Scripts
-	scripts = get_scripts()
+	if sys.platform == 'darwin':
+		scripts = get_scripts(excludes=[appname + '-apply-profiles'])
+	else:
+		scripts = get_scripts()
 	# Doc files
 	data_files = []
 	if not is_rpm_build or doc_layout.startswith("deb"):
@@ -482,6 +520,18 @@ def setup():
 		data_files += get_data(data, "xtra_package_data", name, sys.platform)
 		if sys.platform == "win32":
 			data_files.extend([(os.path.join(data, "lib"), [sys.executable])])
+			# OpenAL DLLs for pyglet
+			openal32 = ctypes.util.find_library("OpenAL32.dll")
+			wrap_oal = ctypes.util.find_library("wrap_oal.dll")
+			if openal32:
+				oal = [openal32]
+				if wrap_oal:
+					oal.append(wrap_oal)
+				else:
+					print "WARNING: wrap_oal.dll not found!"
+				data_files.append((data, oal))
+			else:
+				print "WARNING: OpenAL32.dll not found!"
 		elif sys.platform != "darwin":
 			# Linux
 			data_files.append((os.path.join(os.path.dirname(data), "appdata"),
@@ -949,6 +999,7 @@ setup(ext_modules=[Extension("%s.lib%s.RealDisplaySizeMM", sources=%r,
 					name + "-3DLUT-maker.desktop",
 					name + "-curve-viewer.desktop",
 					name + "-profile-info.desktop",
+					name + "-scripting-client.desktop",
 					name + "-synthprofile.desktop",
 					name + "-testchart-editor.desktop",
 					"pnp.ids",
