@@ -11,6 +11,8 @@ import re
 import sys
 if sys.platform == "win32":
 	from ctypes import windll
+elif sys.platform == "darwin":
+	from platform import mac_ver
 
 from wxaddons import wx
 from lib.agw import labelbook
@@ -22,8 +24,10 @@ from config import enc
 from log import get_file_logger
 from meta import name as appname
 from ordereddict import OrderedDict
+from util_list import intlist
 from util_str import safe_unicode, wrap
-from wxwindows import FlatShadedButton, numpad_keycodes
+from wxwindows import BaseApp, BaseFrame, FlatShadedButton, numpad_keycodes
+import audio
 import config
 import localization as lang
 
@@ -115,7 +119,7 @@ class DisplayAdjustmentImageContainer(labelbook.ImageContainer):
 		
 			if self._pagesInfoVec[i].GetPosition() == wx.Point(-1, -1):
 				break
-			if getcfg("measurement_mode") != "c" and i in self.GetParent().disabled_pages:
+			if i in self.GetParent().disabled_pages:
 				continue
 			
 			# For Web Hover style, we test the TextRect
@@ -198,7 +202,7 @@ class DisplayAdjustmentImageContainer(labelbook.ImageContainer):
 		for i in xrange(len(self._pagesInfoVec)):
 			if self.GetParent().GetParent().is_busy and i != self.GetParent().GetSelection():
 				continue
-			if getcfg("measurement_mode") != "c" and i in self.GetParent().disabled_pages:
+			if i in self.GetParent().disabled_pages:
 				continue
 
 			count = count + 1
@@ -431,7 +435,7 @@ class DisplayAdjustmentFlatImageBook(labelbook.FlatImageBook):
 			self.ResizeTabArea()
 			
 		self._mainSizer.Layout()
-		dummy = wx.SizeEvent()
+		dummy = wx.SizeEvent(wx.DefaultSize)
 		wx.PostEvent(self, dummy)
 		self._pages.Refresh()
 
@@ -547,9 +551,8 @@ class DisplayAdjustmentPanel(wx.Panel):
 
 	def add_marker(self, direction="top"):
 		self.sizer.Add((1, 1))
-		self.sizer.Add(wx.StaticBitmap(self,
-									   bitmap=getbitmap("theme/marker_%s" %
-														direction),
+		self.sizer.Add(wx.StaticBitmap(self, -1,
+									   getbitmap("theme/marker_%s" % direction),
 									   size=(200, 10)))
 	
 	def add_txt(self, name, spacer=None, border=8):
@@ -594,16 +597,17 @@ class DisplayAdjustmentPanel(wx.Panel):
 							   lang.getstr("calibration.interactive_display_adjustment.generic_hint.singular"))
 			self.desc.Wrap(250)
 
-class DisplayAdjustmentFrame(wx.Frame):
+class DisplayAdjustmentFrame(BaseFrame):
 
 	def __init__(self, parent=None, handler=None,
 				 keyhandler=None, start_timer=True):
-		wx.Frame.__init__(self, parent, wx.ID_ANY,
+		BaseFrame.__init__(self, parent, wx.ID_ANY,
 						  lang.getstr("calibration.interactive_display_adjustment"),
-						  style=wx.DEFAULT_FRAME_STYLE | wx.TAB_TRAVERSAL)
+						  style=wx.DEFAULT_FRAME_STYLE | wx.TAB_TRAVERSAL,
+						  name="displayadjustmentframe")
 		self.SetIcons(get_icon_bundle([256, 48, 32, 16], appname))
 		self.SetBackgroundColour(BGCOLOUR)
-		self.sizer = wx.FlexGridSizer(0, 3)
+		self.sizer = wx.FlexGridSizer(0, 3, 0, 0)
 		self.sizer.AddGrowableCol(1)
 		self.sizer.AddGrowableRow(2)
 		self.SetSizer(self.sizer)
@@ -671,10 +675,7 @@ class DisplayAdjustmentFrame(wx.Frame):
 		
 		# Sound when measuring
 		# Needs to be stereo!
-		try:
-			self.measurement_sound = wx.Sound(get_data_path("beep.wav") or "")
-		except NotImplementedError:
-			pass
+		self.measurement_sound = audio.Sound(get_data_path("beep.wav"))
 		
 		# Add buttons
 		self.btnsizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -846,17 +847,21 @@ class DisplayAdjustmentFrame(wx.Frame):
 		self.target_br = None
 		self._assign_image_list()
 		if getcfg("measurement_mode") == "c":
-			self.lb.disabled_pages = tuple()
-			self.lb.SetSelection(0)
+			self.lb.disabled_pages = []
+			if getcfg("calibration.black_luminance", False):
+				self.lb.SetSelection(0)
+			else:
+				self.lb.disabled_pages.append(0)
+				self.lb.SetSelection(1)
 		else:
-			self.lb.disabled_pages = (0, 3)
+			self.lb.disabled_pages = [0, 3]
 			self.lb.SetSelection(1)
 		if getcfg("trc"):
-			self.calibration_btn.SetLabel(" " + lang.getstr("calibration.start"))
+			self.calibration_btn.SetLabel(lang.getstr("calibration.start"))
 		elif getcfg("calibration.continue_next"):
-			self.calibration_btn.SetLabel(" " + lang.getstr("calibration.skip"))
+			self.calibration_btn.SetLabel(lang.getstr("calibration.skip"))
 		else:
-			self.calibration_btn.SetLabel(" " + lang.getstr("finish"))
+			self.calibration_btn.SetLabel(lang.getstr("finish"))
 		for btn in (self.adjustment_btn, self.sound_on_off_btn,
 					self.calibration_btn):
 			if hasattr(btn, "_lastBestSize"):
@@ -887,7 +892,12 @@ class DisplayAdjustmentFrame(wx.Frame):
 			page = self.lb.GetPage(pagenum)
 			page.SetSize((w, -1))
 			page.desc.SetLabel(page.desc.GetLabel().replace("\n", " "))
-			page.desc.Wrap(w - 12)
+			if (sys.platform == "darwin" and
+				intlist(mac_ver()[0].split(".")) >= [10, 10]):
+				margin = 24
+			else:
+				margin = 12
+			page.desc.Wrap(w - margin)
 			bitmaps = {"black_level": {CRT: "luminance",
 									   not CRT: "black_level"},
 					   "luminance": {CRT: "contrast",
@@ -949,14 +959,12 @@ class DisplayAdjustmentFrame(wx.Frame):
 		if getattr(self, "adjustment_btn", None):
 			self.adjustment_btn._bitmap = getbitmap("theme/icons/10x10/%s" %
 													icon)
-			self.adjustment_btn.SetLabel(" " +
-										 lang.getstr("calibration.interactive_display_adjustment.%s" %
+			self.adjustment_btn.SetLabel(lang.getstr("calibration.interactive_display_adjustment.%s" %
 													 startstop))
 			self.adjustment_btn.Enable(enable)
 			return
 		self.adjustment_btn = self.create_gradient_button(getbitmap("theme/icons/10x10/%s" %
 																	icon),
-														  " " +
 														  lang.getstr("calibration.interactive_display_adjustment.%s" %
 																	  startstop),
 														  name="adjustment_btn")
@@ -1220,11 +1228,8 @@ class DisplayAdjustmentFrame(wx.Frame):
 				set_label_and_size(self.lb.GetCurrentPage().txt["black_point"], label)
 		if ((current_br or current_bl or xy_dE_rgb) and
 			self.lb.GetCurrentPage().ctrltype != "check_all"):
-			if (getattr(self, "measurement_sound", None) and
-				#self.measurement_play_sound_ctrl.GetValue() and
-				getcfg("measurement.play_sound") and
-				self.measurement_sound.IsOk()):
-				self.measurement_sound.Play(wx.SOUND_ASYNC)
+			if getcfg("measurement.play_sound"):
+				self.measurement_sound.safe_play()
 			self.indicator_ctrl.SetBitmap(indicator)
 			self.btnsizer.Layout()
 		if current_br or current_bl or xy_dE_rgb or white_xy_dE or black_xy_dE:
@@ -1305,14 +1310,14 @@ if __name__ == "__main__":
 			return True
 	config.initcfg()
 	lang.init()
-	app = wx.App(0)
+	app = BaseApp(0)
 	if "--crt" in sys.argv[1:]:
 		setcfg("measurement_mode", "c")
 	else:
 		setcfg("measurement_mode", "l")
-	frame = DisplayAdjustmentFrame(start_timer=False)
-	frame.worker = Worker()
-	frame.Show()
+	app.TopWindow = DisplayAdjustmentFrame(start_timer=False)
+	app.TopWindow.worker = Worker()
+	app.TopWindow.Show()
 	i = 0
 	def test(bytes=None):
 		global i
@@ -1570,7 +1575,7 @@ Press 1 .. 7"""][i]
 					i += 1
 				else:
 					i -= 4
-				wx.CallAfter(frame.reset)
+				wx.CallAfter(app.TopWindow.reset)
 			txt = [r"""Setting up the instrument
 Place instrument on test window.
 Hit Esc or Q to give up, any other key to continue:
@@ -1614,13 +1619,13 @@ Target advertised gamma = 2.400000"""][i] + r"""
 
 Display adjustment menu:""" + menu
 		elif bytes == "8":
-			wx.CallAfter(frame.Close)
+			wx.CallAfter(app.TopWindow.Close)
 			return
 		else:
 			return
 		for line in txt.split("\n"):
 			sleep(.0625)
-			wx.CallAfter(frame.write, line)
+			wx.CallAfter(app.TopWindow.write, line)
 			print line
 	start_new_thread(test, tuple())
 	app.MainLoop()
