@@ -22,7 +22,7 @@ from meta import name as appname, version as appversion
 from util_os import launch_file, waccess
 from wxaddons import CustomEvent
 from wxMeasureFrame import MeasureFrame
-from wxwindows import FlatShadedButton, numpad_keycodes
+from wxwindows import BaseApp, BaseFrame, FlatShadedButton, numpad_keycodes
 import colormath
 import config
 import localization as lang
@@ -48,16 +48,17 @@ class FlatShadedNumberedButton(FlatShadedButton):
 		self.index = index
 
 
-class DisplayUniformityFrame(wx.Frame):
+class DisplayUniformityFrame(BaseFrame):
 
 	def __init__(self, parent=None, handler=None,
 				 keyhandler=None, start_timer=True, rows=3, cols=5):
-		wx.Frame.__init__(self, parent, wx.ID_ANY,
+		BaseFrame.__init__(self, parent, wx.ID_ANY,
 						  lang.getstr("report.uniformity"),
-						  style=wx.DEFAULT_FRAME_STYLE | wx.TAB_TRAVERSAL)
+						  style=wx.DEFAULT_FRAME_STYLE | wx.TAB_TRAVERSAL,
+						  name="displayuniformityframe")
 		self.SetIcons(get_icon_bundle([256, 48, 32, 16], appname))
 		self.SetBackgroundColour(BGCOLOUR)
-		self.sizer = wx.GridSizer(rows, cols)
+		self.sizer = wx.GridSizer(rows, cols, 0, 0)
 		self.SetSizer(self.sizer)
 		
 		self.rows = rows
@@ -73,8 +74,8 @@ class DisplayUniformityFrame(wx.Frame):
 			sizer = wx.BoxSizer(wx.VERTICAL)
 			panel.SetSizer(sizer)
 			self.panels.append(panel)
-			button = FlatShadedNumberedButton(panel, label=" %s " %
-													 lang.getstr("measure"),
+			button = FlatShadedNumberedButton(panel,
+											  label=lang.getstr("measure"),
 											  bitmap=getbitmap("theme/icons/10x10/record"),
 											  bgcolour=LIGHTGRAY,
 											  fgcolour=BLACK, index=index)
@@ -280,7 +281,7 @@ class DisplayUniformityFrame(wx.Frame):
 					display_no, geometry, client_area = self.get_display()
 					# Translate from wx display index to Argyll display index
 					geometry = "%i, %i, %ix%i" % tuple(geometry)
-					for i, display in enumerate(getcfg("displays").split(os.pathsep)):
+					for i, display in enumerate(getcfg("displays")):
 						if display.find("@ " + geometry) > -1:
 							safe_print("Found display %s at index %i" % 
 									   (display, i))
@@ -328,7 +329,7 @@ class DisplayUniformityFrame(wx.Frame):
 						show_result_dialog(exception, self)
 					else:
 						launch_file(save_path)
-				if getcfg("measurement.continuous"):
+				if getcfg("uniformity.measure.continuous"):
 					self.measure(event=Event(self.buttons[self.index]))
 	
 	def reset(self):
@@ -403,10 +404,10 @@ if __name__ == "__main__":
 	config.initcfg()
 	lang.init()
 	lang.update_defaults()
-	app = wx.App(0)
-	frame = DisplayUniformityFrame(start_timer=False, cols=3)
-	frame.worker = Worker()
-	frame.Show()
+	app = BaseApp(0)
+	app.TopWindow = DisplayUniformityFrame(start_timer=False, cols=3)
+	app.TopWindow.worker = Worker()
+	app.TopWindow.Show()
 	i = 0
 	def test(bytes=None):
 		global i
@@ -914,19 +915,19 @@ and hit [A-Z] to read white and setup FWA compensation (keyed to letter)
 [a-z] to read and make FWA compensated reading from keyed reference
 'r' to set reference, 's' to save spectrum,
 'h' to toggle high res., 'k' to do a calibration
-Hit ESC or Q to exit, any other key to take a reading:"""]][frame.index][i]
+Hit ESC or Q to exit, any other key to take a reading:"""]][app.TopWindow.index][i]
 			if i < 3:
 				i += 1
 			else:
 				i -= 3
 		elif bytes in ("Q", "q"):
-			wx.CallAfter(frame.Close)
+			wx.CallAfter(app.TopWindow.Close)
 			return
 		else:
 			return
 		for line in txt.split("\n"):
 			sleep(.03125)
-			wx.CallAfter(frame.write, line)
+			wx.CallAfter(app.TopWindow.write, line)
 			print line
 	start_new_thread(test, tuple())
 	app.MainLoop()
