@@ -9,9 +9,9 @@ import tempfile
 
 from argyll_cgats import cal_to_fake_profile, vcgt_to_cal
 from colormath import interp
-from config import (fs_enc, get_argyll_display_number, get_display_profile,
-					get_display_rects, getcfg, geticon, get_verified_path,
-					setcfg)
+from config import (fs_enc, get_argyll_display_number, get_data_path,
+					get_display_profile, get_display_rects, getcfg, geticon,
+					get_verified_path, setcfg)
 from log import safe_print
 from meta import name as appname
 from options import debug
@@ -22,8 +22,8 @@ from worker import (Error, UnloggedError, UnloggedInfo, Worker, get_argyll_util,
 					make_argyll_compatible_path, show_result_dialog)
 from wxaddons import get_platform_window_decoration_size, wx
 from wxMeasureFrame import MeasureFrame
-from wxwindows import (BitmapBackgroundPanelText, CustomCheckBox, FileDrop,
-					   InfoDialog, TooltipWindow)
+from wxwindows import (BaseApp, BaseFrame, BitmapBackgroundPanelText,
+					   CustomCheckBox, FileDrop, InfoDialog, TooltipWindow)
 from wxfixes import GenBitmapButton as BitmapButton
 import colormath
 import config
@@ -52,7 +52,8 @@ class CoordinateType(list):
 	
 	"""
 	
-	def __init__(self):
+	def __init__(self, profile=None):
+		self.profile = profile
 		self._transfer_function = {}
 	
 	def get_gamma(self, use_vmin_vmax=False, average=True, least_squares=False,
@@ -87,7 +88,7 @@ class CoordinateType(list):
 		transfer_function = self._transfer_function.get((best, slice))
 		if transfer_function:
 			return transfer_function
-		trc = CoordinateType()
+		trc = ICCP.CurveType()
 		match = {}
 		vmin = self[0][0]
 		vmax = self[-1][0]
@@ -97,11 +98,32 @@ class CoordinateType(list):
 				best_yx = (y, x)
 		gamma = colormath.get_gamma([(best_yx[1] / 255.0 * 100.0, best_yx[0])], 100.0, vmin, vmax)
 		for name, exp in (("Rec. 709", -709),
+						  ("Rec. 1886", -1886),
 						  ("SMPTE 240M", -240),
+						  ("SMPTE 2084", -2084),
+						  ("DICOM", -1023),
 						  ("L*", -3.0),
 						  ("sRGB", -2.4),
 						  ("Gamma %.2f" % gamma, gamma)):
-			trc.set_trc(exp, self, vmin, vmax)
+			if name in ("DICOM", "Rec. 1886", "SMPTE 2084"):
+				if self.profile and isinstance(self.profile.tags.get("lumi"),
+											   ICCP.XYZType):
+					white_cdm2 = self.profile.tags.lumi.Y
+				else:
+					white_cdm2 = 100.0
+				black_Y = vmin / 100.0
+				black_cdm2 = black_Y * white_cdm2
+				try:
+					if name == "DICOM":
+						trc.set_dicom_trc(black_cdm2, white_cdm2, size=len(self))
+					elif name == "Rec. 1886":
+						trc.set_bt1886_trc(black_Y, size=len(self))
+					elif name == "SMPTE 2084":
+						trc.set_smpte2084_trc(black_cdm2, white_cdm2, size=len(self))
+				except ValueError:
+					continue
+			else:
+				trc.set_trc(exp, len(self), vmin * 655.35, vmax * 655.35)
 			match[(name, exp)] = 0.0
 			count = 0
 			start = slice[0] * 255
@@ -113,7 +135,7 @@ class CoordinateType(list):
 					if n:
 						n = n[0]
 						##n2 = colormath.XYZ2Lab(0, trc[i][0], 0)[0]
-						n2 = colormath.get_gamma([(trc[i][1] / 255.0 * 100, trc[i][0])], 100.0, vmin, vmax, False)
+						n2 = colormath.get_gamma([(i / (len(trc) - 1.0) * 65535, trc[i])], 65535, vmin * 655.35, vmax * 655.35, False)
 						if n2:
 							n2 = n2[0]
 							match[(name, exp)] += 1 - (max(n, n2) - min(n, n2)) / n2
@@ -367,9 +389,14 @@ class LUTCanvas(plot.PlotCanvas):
 			else:
 				r_points = g_points = b_points = linear_points
 		
-		self.r_unique = len(set(round(y) for x, y in r_points))
-		self.g_unique = len(set(round(y) for x, y in g_points))
-		self.b_unique = len(set(round(y) for x, y in b_points))
+		# Note: We need to make sure each point is a float because it
+		# might be a decimal.Decimal, which can't be divided by floats!
+		self.r_unique = len(set(round(float(y) / axis_y * irange[-1])
+								for x, y in r_points))
+		self.g_unique = len(set(round(float(y) / axis_y * irange[-1])
+								for x, y in g_points))
+		self.b_unique = len(set(round(float(y) / axis_y * irange[-1])
+								for x, y in b_points))
 
 		legend = []
 		colour = None
@@ -404,7 +431,9 @@ class LUTCanvas(plot.PlotCanvas):
 		linear_points = [(i, int(round(v / axis_y * maxv))) for i, v in
 						 linear_points]
 		if colour and points:
-			points_quantized = [(i, int(round(v / axis_y * maxv))) for i, v in
+			# Note: We need to make sure each point is a float because it
+			# might be a decimal.Decimal, which can't be divided by floats!
+			points_quantized = [(i, int(round(float(v) / axis_y * maxv))) for i, v in
 								points]
 			suffix = ((', ' + lang.getstr('linear').capitalize()) if 
 						points_quantized == (linear if detect_increments 
@@ -413,21 +442,27 @@ class LUTCanvas(plot.PlotCanvas):
 							  colour=colour))
 		if colour != 'white':
 			if r and colour not in ('yellow', 'magenta'):
-				points_quantized = [(i, int(round(v / axis_y * maxv)))
+				# Note: We need to make sure each point is a float because it
+				# might be a decimal.Decimal, which can't be divided by floats!
+				points_quantized = [(i, int(round(float(v) / axis_y * maxv)))
 									for i, v in r_points]
 				suffix = ((', ' + lang.getstr('linear').capitalize()) if 
 							points_quantized == (linear if detect_increments 
 												 else linear_points) else '')
 				lines.append(Plot(r_points, legend='R' + suffix, colour='red'))
 			if g and colour not in ('yellow', 'cyan'):
-				points_quantized = [(i, int(round(v / axis_y * maxv)))
+				# Note: We need to make sure each point is a float because
+				# it might be a decimal.Decimal, which can't be divided by floats!
+				points_quantized = [(i, int(round(float(v) / axis_y * maxv)))
 									for i, v in g_points]
 				suffix = ((', ' + lang.getstr('linear').capitalize()) if 
 							points_quantized == (linear if detect_increments 
 												 else linear_points) else '')
 				lines.append(Plot(g_points, legend='G' + suffix, colour='green'))
 			if b and colour not in ('cyan', 'magenta'):
-				points_quantized = [(i, int(round(v / axis_y * maxv)))
+				# Note: We need to make sure each point is a float because
+				# it might be a decimal.Decimal, which can't be divided by floats!
+				points_quantized = [(i, int(round(float(v) / axis_y * maxv)))
 									for i, v in b_points]
 				suffix = ((', ' + lang.getstr('linear').capitalize()) if 
 							points_quantized == (linear if detect_increments 
@@ -452,12 +487,12 @@ class LUTCanvas(plot.PlotCanvas):
 			return
 		graphics, xAxis, yAxis= self.last_draw
 		# sizes axis to axis type, create lower left and upper right corners of plot
-		if xAxis == None or yAxis == None:
+		if xAxis is None or yAxis is None:
 			# One or both axis not specified in Draw
 			p1, p2 = graphics.boundingBox()     # min, max points of graphics
-			if xAxis == None:
+			if xAxis is None:
 				xAxis = self._axisInterval(self._xSpec, p1[0], p2[0]) # in user units
-			if yAxis == None:
+			if yAxis is None:
 				yAxis = self._axisInterval(self._ySpec, p1[1], p2[1])
 			# Adjust bounding box for axis spec
 			p1[0],p1[1] = xAxis[0], yAxis[0]     # lower left corner user scale (xmin,ymin)
@@ -470,7 +505,7 @@ class LUTCanvas(plot.PlotCanvas):
 		# allow graph to overlap axis lines by adding units to width and height
 		dc.SetClippingRegion(ptx,pty,rectWidth+2,rectHeight+2)
 
-		if sys.platform == "darwin":
+		if sys.platform == "darwin" or "gtk3" in wx.PlatformInfo:
 			color = wx.WHITE
 		else:
 			color = wx.BLACK
@@ -491,7 +526,7 @@ class LUTCanvas(plot.PlotCanvas):
 			if pointScaled == True based on screen coords
 			if pointScaled == False based on user coords
 		"""
-		if self.last_draw == None:
+		if self.last_draw is None:
 			#no graph available
 			return []
 		graphics, xAxis, yAxis= self.last_draw
@@ -679,14 +714,16 @@ class LUTCanvas(plot.PlotCanvas):
 			self._DrawCanvas(self.last_draw[0])
 
 
-class LUTFrame(wx.Frame):
+class LUTFrame(BaseFrame):
 
 	def __init__(self, *args, **kwargs):
 	
 		if len(args) < 3 and not "title" in kwargs:
 			kwargs["title"] = lang.getstr("calibration.lut_viewer.title")
+		if not "name" in kwargs:
+			kwargs["name"] = "lut_viewer"
 		
-		wx.Frame.__init__(self, *args, **kwargs)
+		BaseFrame.__init__(self, *args, **kwargs)
 		
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16],
 											 appname + "-curve-viewer"))
@@ -812,7 +849,7 @@ class LUTFrame(wx.Frame):
 		
 		self.box_sizer.Add((0, 0))
 		
-		self.cbox_sizer.Add((16, 0))
+		self.cbox_sizer.Add((10, 0))
 
 		self.reload_vcgt_btn = BitmapButton(self.box_panel, -1,
 											geticon(16, "stock_refresh"),
@@ -921,6 +958,7 @@ class LUTFrame(wx.Frame):
 
 		children = self.GetAllChildren()
 
+		self.Bind(wx.EVT_KEY_DOWN, self.key_handler)
 		for child in children:
 			if isinstance(child, wx.Choice):
 				child.SetMaxFontSize(11)
@@ -1095,7 +1133,7 @@ class LUTFrame(wx.Frame):
 			(self.worker.argyll_version[0:3] > [1, 1, 0] or
 			 (self.worker.argyll_version[0:3] == [1, 1, 0] and
 			  not "Beta" in self.worker.argyll_version_string)) and
-			not config.get_display_name() in config.untethered_displays):
+			not config.is_untethered_display()):
 			tmp = self.worker.create_tempdir()
 			if isinstance(tmp, Exception):
 				show_result_dialog(tmp, self)
@@ -1153,47 +1191,47 @@ class LUTFrame(wx.Frame):
 		if (intent == "r" and (not ("B2A0" in self.profile.tags or
 									"A2B0" in self.profile.tags) or
 							   not self.toggle_clut.GetValue()) and
-			isinstance(self.profile.tags.get("rTRC"), ICCP.CurveType) and
-			isinstance(self.profile.tags.get("gTRC"), ICCP.CurveType) and
-			isinstance(self.profile.tags.get("bTRC"), ICCP.CurveType)):
+			isinstance(self.rTRC, ICCP.CurveType) and
+			isinstance(self.gTRC, ICCP.CurveType) and
+			isinstance(self.bTRC, ICCP.CurveType)):
 			#self.rTRC = self.profile.tags.rTRC
 			#self.gTRC = self.profile.tags.gTRC
 			#self.bTRC = self.profile.tags.bTRC
 			#return
 			# Use TRC tags if no LUT
-			if (len(self.profile.tags.rTRC) == 1 and
-				len(self.profile.tags.gTRC) == 1 and
-				len(self.profile.tags.bTRC) == 1):
+			if (len(self.rTRC) == 1 and
+				len(self.gTRC) == 1 and
+				len(self.bTRC) == 1):
 				# Gamma, convert to curves
 				trc = {"rTRC": [],
 					   "gTRC": [],
 					   "bTRC": []}
 				for sig in ("rTRC", "gTRC", "bTRC"):
-					gamma = self.profile.tags[sig][0]
-					setattr(self, sig, CoordinateType())
+					gamma = getattr(self, sig)[0]
+					setattr(self, sig, CoordinateType(self.profile))
 					for i in xrange(256):
 						trc[sig].append(math.pow(i / 255.0, gamma) * 65535)
 			else:
-				trc = {"rTRC": self.profile.tags.rTRC,
-					   "gTRC": self.profile.tags.gTRC,
-					   "bTRC": self.profile.tags.bTRC}
+				trc = {"rTRC": self.rTRC,
+					   "gTRC": self.gTRC,
+					   "bTRC": self.bTRC}
 			# Curves
 			for sig in ("rTRC", "gTRC", "bTRC"):
 				x, xp, y, yp = [], [], [], []
 				# First, get actual values
 				for i, Y in enumerate(trc[sig]):
-					if not i or Y > trc[sig][i - 1]:
-						xp.append(i / (len(trc[sig]) - 1.0) * 255)
-						yp.append(Y / 65535.0 * 100)
-				# Second, interpolate to given size and use the same Y axis 
+					##if not i or Y >= trc[sig][i - 1]:
+					xp.append(i / (len(trc[sig]) - 1.0) * 255)
+					yp.append(Y / 65535.0 * 100)
+				# Second, interpolate to given size and use the same y axis 
 				# for all channels
 				for i in xrange(size):
 					x.append(i / (size - 1.0) * 255)
 					y.append(colormath.Lab2XYZ(i / (size - 1.0) * 100, 0, 0)[1] * 100)
-				yi = interp(x, xp, yp)
-				xi = interp(y, yi, x)
-				setattr(self, sig, CoordinateType())
-				for v, Y in zip(xi, y):
+				xi = interp(y, yp, xp)
+				yi = interp(x, xi, y)
+				setattr(self, sig, CoordinateType(self.profile))
+				for Y, v in zip(yi, x):
 					if Y <= yp[0]:
 						Y = yp[0]
 					getattr(self, sig).append([Y, v])
@@ -1271,14 +1309,16 @@ class LUTFrame(wx.Frame):
 		else:
 			Lab_triplets = odata
 
-		self.rTRC = CoordinateType()
-		self.gTRC = CoordinateType()
-		self.bTRC = CoordinateType()
+		self.rTRC = CoordinateType(self.profile)
+		self.gTRC = CoordinateType(self.profile)
+		self.bTRC = CoordinateType(self.profile)
 		for j, RGB in enumerate(RGB_triplets):
 			for i, v in enumerate(RGB):
 				v = min(v, 1.0)
 				v *= 255
 				X, Y, Z = colormath.Lab2XYZ(*Lab_triplets[j], **{"scale": 100})
+				if not v:
+					continue
 				if direction in ("b", "if"):
 					X = Z = Y
 				elif intent == "a":
@@ -1290,6 +1330,25 @@ class LUTFrame(wx.Frame):
 					self.gTRC.append([Y, v])
 				elif i == 2:
 					self.bTRC.append([Z, v])
+		for sig in ("rTRC", "gTRC", "bTRC"):
+			x, xp, y, yp = [], [], [], []
+			# First, get actual values
+			for i, (Y, v) in enumerate(getattr(self, sig)):
+				##if not i or Y >= trc[sig][i - 1]:
+				xp.append(v)
+				yp.append(Y)
+			# Second, interpolate to given size and use the same y axis 
+			# for all channels
+			for i in xrange(size):
+				x.append(i / (size - 1.0) * 255)
+				y.append(colormath.Lab2XYZ(i / (size - 1.0) * 100, 0, 0)[1] * 100)
+			xi = interp(y, yp, xp)
+			yi = interp(x, xi, y)
+			setattr(self, sig, CoordinateType(self.profile))
+			for Y, v in zip(yi, x):
+				if Y <= yp[0]:
+					Y = yp[0]
+				getattr(self, sig).append([Y, v])
 
 	def move_handler(self, event):
 		if not self.IsShownOnScreen():
@@ -1310,6 +1369,27 @@ class LUTFrame(wx.Frame):
 		self.client.resetzoom()
 		self.DrawLUT()
 		wx.CallAfter(self.client.center)
+
+	def get_commands(self):
+		return self.get_common_commands() + ["curve-viewer [filename]",
+											 "load <filename>"]
+
+	def process_data(self, data):
+		if (data[0] == "curve-viewer" and
+			len(data) < 3) or (data[0] == "load" and len(data) == 2):
+			if self.IsIconized():
+				self.Restore()
+			self.Raise()
+			if len(data) == 2:
+				path = data[1]
+				if not os.path.isfile(path) and not os.path.isabs(path):
+					path = get_data_path(path)
+				if not path:
+					return "fail"
+				else:
+					self.droptarget.OnDropFiles(0, 0, [path])
+			return "ok"
+		return "invalid"
 	
 	def reload_vcgt_handler(self, event):
 		cmd, args = self.worker.prepare_dispwin(True)
@@ -1347,9 +1427,9 @@ class LUTFrame(wx.Frame):
 			title = lang.getstr("calibration.lut_viewer.title")
 		self.SetTitle(title)
 		self.profile = profile
-		self.rTRC = profile.tags.get("rTRC")
-		self.gTRC = profile.tags.get("gTRC")
-		self.bTRC = profile.tags.get("bTRC")
+		self.rTRC = profile.tags.get("rTRC", profile.tags.get("kTRC"))
+		self.gTRC = profile.tags.get("gTRC", profile.tags.get("kTRC"))
+		self.bTRC = profile.tags.get("bTRC", profile.tags.get("kTRC"))
 		self.trc = None
 		curves = []
 		curves.append(lang.getstr('vcgt'))
@@ -1362,18 +1442,22 @@ class LUTFrame(wx.Frame):
 			try:
 				self.lookup_tone_response_curves()
 			except Exception, exception:
-				show_result_dialog(exception, self)
+				wx.CallAfter(show_result_dialog, exception, self)
 			else:
 				curves.append(lang.getstr('[rgb]TRC'))
 		selection = self.plot_mode_select.GetSelection()
+		center = False
 		if curves and (selection < 0 or selection > len(curves) - 1):
 			selection = 0
+			center = True
 		self.plot_mode_select.SetItems(curves)
 		self.plot_mode_select.Enable(len(curves) > 1)
 		self.plot_mode_select.SetSelection(selection)
 		self.cbox_sizer.Layout()
 		self.box_sizer.Layout()
 		self.DrawLUT()
+		if center:
+			wx.CallAfter(self.client.center)
 		wx.CallAfter(self.handle_errors)
 
 	def add_tone_values(self, legend):
@@ -1438,13 +1522,13 @@ class LUTFrame(wx.Frame):
 			if (not getattr(self, "trc", None) and
 				len(self.rTRC) == len(self.gTRC) == len(self.bTRC)):
 				if isinstance(self.rTRC, ICCP.CurveType):
-					self.trc = ICCP.CurveType()
+					self.trc = ICCP.CurveType(profile=self.profile)
 					for i in xrange(len(self.rTRC)):
 						self.trc.append((self.rTRC[i] +
 										 self.gTRC[i] +
 										 self.bTRC[i]) / 3.0)
 				else:
-					self.trc = CoordinateType()
+					self.trc = CoordinateType(self.profile)
 					for i in xrange(len(self.rTRC)):
 						self.trc.append([(self.rTRC[i][0] +
 										  self.gTRC[i][0] +
@@ -1550,9 +1634,12 @@ class LUTFrame(wx.Frame):
 							  ("B2A0" in self.profile.tags or
 							   "A2B0" in self.profile.tags))
 		self.toggle_clut.Enable(self.plot_mode_select.GetSelection() == 1 and
-								isinstance(self.profile.tags.get("rTRC"), ICCP.CurveType) and
-								isinstance(self.profile.tags.get("gTRC"), ICCP.CurveType) and
-								isinstance(self.profile.tags.get("bTRC"), ICCP.CurveType))
+								isinstance(self.profile.tags.get("rTRC"),
+										   ICCP.CurveType) and
+								isinstance(self.profile.tags.get("gTRC"),
+										   ICCP.CurveType) and
+								isinstance(self.profile.tags.get("bTRC"),
+										   ICCP.CurveType))
 		self.save_plot_btn.Enable(bool(curves))
 		if hasattr(self, "reload_vcgt_btn"):
 			self.reload_vcgt_btn.Enable(not(self.plot_mode_select.GetSelection()) and
@@ -1602,6 +1689,7 @@ class LUTFrame(wx.Frame):
 		self.Thaw()
 
 	def OnClose(self, event):
+		self.listening = False
 		if self.worker.tempdir and os.path.isdir(self.worker.tempdir):
 			self.worker.wrapup(False)
 		wx.GetApp().ExitMainLoop()
@@ -1793,49 +1881,43 @@ class LUTFrame(wx.Frame):
 		self.show_actual_lut_cb.Enable((self.worker.argyll_version[0:3] > [1, 1, 0] or
 										(self.worker.argyll_version[0:3] == [1, 1, 0] and
 										 not "Beta" in self.worker.argyll_version_string)) and
-									   not config.get_display_name() in
-									   config.untethered_displays)
+									   not config.is_untethered_display())
 		self.show_actual_lut_cb.SetValue(bool(getcfg("lut_viewer.show_actual_lut")) and
-										 not config.get_display_name() in
-										 config.untethered_displays)
+										 not config.is_untethered_display())
 	
 	@property
 	def worker(self):
 		return self.client.worker
 
 
-class LUTViewer(wx.App):
-
-	def OnInit(self):
-		self.frame = LUTFrame(None, -1)
-		self.frame.Bind(wx.EVT_CLOSE, self.frame.OnClose, self.frame)
-		return True
-
-
-def main(profile=None):
-	config.initcfg()
+def main():
+	config.initcfg("curve-viewer")
 	# Backup display config
 	cfg_display = getcfg("display.number")
 	lang.init()
 	lang.update_defaults()
-	app = LUTViewer(0)
-	app.frame.worker.enumerate_displays_and_ports(check_lut_access=False,
-												  enumerate_ports=False)
-	if profile and profile.startswith("-"):
-		profile = None
-	app.frame.display_no, geometry, client_area = app.frame.get_display()
-	app.frame.Bind(wx.EVT_MOVE, app.frame.move_handler, app.frame)
+	app = BaseApp(0)
+	app.TopWindow = LUTFrame(None, -1)
+	app.TopWindow.Bind(wx.EVT_CLOSE, app.TopWindow.OnClose, app.TopWindow)
+	if sys.platform == "darwin":
+		app.TopWindow.init_menubar()
+	app.TopWindow.worker.enumerate_displays_and_ports(check_lut_access=False,
+													  enumerate_ports=False)
+	app.TopWindow.display_no, geometry, client_area = app.TopWindow.get_display()
+	app.TopWindow.Bind(wx.EVT_MOVE, app.TopWindow.move_handler, app.TopWindow)
 	display_no = get_argyll_display_number(geometry)
 	setcfg("display.number", display_no + 1)
-	app.frame.update_controls()
-	if profile:
-		app.frame.drop_handler(profile)
+	app.TopWindow.update_controls()
+	app.TopWindow.listen()
+	for arg in sys.argv[1:]:
+		if os.path.isfile(arg):
+			app.TopWindow.drop_handler(safe_unicode(arg))
+			break
 	else:
-		app.frame.load_lut(get_display_profile(display_no))
-	app.frame.Show()
+		app.TopWindow.load_lut(get_display_profile(display_no))
+	app.TopWindow.Show()
 	app.MainLoop()
-	setcfg("display.number", cfg_display)
-	config.writecfg()
+	config.writecfg(module="curve-viewer", options=("display.number", ))
 
 if __name__ == '__main__':
-	main(*sys.argv[max(len(sys.argv) - 1, 1):])
+	main()
