@@ -17,7 +17,7 @@ from options import debug
 from util_list import floatlist, strlist
 from util_str import safe_unicode
 from wxaddons import wx
-from wxwindows import ConfirmDialog, InfoDialog, InvincibleFrame
+from wxwindows import BaseApp, ConfirmDialog, InfoDialog, InvincibleFrame
 from wxfixes import GenBitmapButton as BitmapButton
 try:
 	import RealDisplaySizeMM as RDSMM
@@ -39,7 +39,7 @@ def get_default_size():
 	"""
 	display_sizes = []
 	display_sizes_mm = []
-	for display_no in xrange(len(getcfg("displays").split(os.pathsep))):
+	for display_no in xrange(len(getcfg("displays"))):
 		display_no = get_display_number(display_no)
 		display_size = wx.Display(display_no).Geometry[2:]
 		display_size_mm = []
@@ -118,19 +118,19 @@ class MeasureFrame(InvincibleFrame):
 	
 	"""
 
-	exitcode = 0
+	exitcode = 1
 
 	def __init__(self, parent=None, id=-1):
 		InvincibleFrame.__init__(self, parent, id, 
 								 lang.getstr("measureframe.title"), 
 								 style=wx.DEFAULT_FRAME_STYLE & 
-									   ~(wx.RESIZE_BORDER | wx.RESIZE_BOX | 
-									     wx.MAXIMIZE_BOX))
+									   ~(wx.RESIZE_BORDER | wx.MAXIMIZE_BOX),
+								 name="measureframe")
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16], appname))
 		self.Bind(wx.EVT_CLOSE, self.close_handler, self)
 		self.Bind(wx.EVT_MOVE, self.move_handler, self)
 		self.panel = wx.Panel(self, -1)
-		self.sizer = wx.GridSizer(3, 1)
+		self.sizer = wx.GridSizer(3, 1, 0, 0)
 		self.panel.SetSizer(self.sizer)
 
 		self.hsizer = wx.BoxSizer(wx.HORIZONTAL)
@@ -404,11 +404,15 @@ class MeasureFrame(InvincibleFrame):
 		self.Hide()
 		if self.Parent:
 			self.Parent.Show()
+			if getattr(self.Parent, "restore_measurement_mode"):
+				self.Parent.restore_measurement_mode()
 			if getattr(self.Parent, "restore_testchart"):
 				self.Parent.restore_testchart()
 		else:
 			writecfg()
 			self.Destroy()
+			if MeasureFrame.exitcode != 255:
+				MeasureFrame.exitcode = 0
 
 	def get_display(self, display_no=None):
 		"""
@@ -538,9 +542,9 @@ class MeasureFrame(InvincibleFrame):
 def main():
 	config.initcfg()
 	lang.init()
-	app = wx.App(0)
-	app.measureframe = MeasureFrame()
-	app.measureframe.Show()
+	app = BaseApp(0)
+	app.TopWindow = MeasureFrame()
+	app.TopWindow.Show()
 	app.MainLoop()
 
 if __name__ == "__main__":
