@@ -17,6 +17,7 @@ from meta import name as appname
 from options import debug
 from ordereddict import OrderedDict
 from util_io import GzipFileProper
+from util_list import intlist
 from util_os import launch_file, make_win32_compatible_long_path, waccess
 from util_str import safe_unicode, strtr, universal_newlines, wrap
 from worker import (Error, UnloggedError, UnloggedInfo, check_set_argyll_bin,
@@ -25,9 +26,10 @@ from worker import (Error, UnloggedError, UnloggedInfo, check_set_argyll_bin,
 from wxaddons import get_platform_window_decoration_size, wx
 from wxLUTViewer import LUTCanvas, LUTFrame
 from wxVRML2X3D import vrmlfile2x3dfile
-from wxwindows import (BitmapBackgroundPanelText, CustomCheckBox, CustomGrid,
-					   CustomRowLabelRenderer, ConfirmDialog,
-					   FileDrop, InfoDialog, SimpleBook, TwoWaySplitter)
+from wxwindows import (BaseApp, BaseFrame, BitmapBackgroundPanelText,
+					   CustomCheckBox, CustomGrid, CustomRowLabelRenderer,
+					   ConfirmDialog, FileDrop, InfoDialog, SimpleBook,
+					   TwoWaySplitter)
 from wxfixes import GenBitmapButton as BitmapButton
 import colormath
 import config
@@ -352,7 +354,7 @@ class GamutCanvas(LUTCanvas):
 					RGBA = colormath.XYZ2RGB(*pcs_triplets[j],
 											 **kwargs)
 					polys.append(plot.PolyMarker([(x, y)],
-												 colour=wx.Colour(*RGBA),
+												 colour=wx.Colour(*intlist(RGBA)),
 												 size=2,
 												 marker="plus",
 												 width=1.75))
@@ -388,7 +390,7 @@ class GamutCanvas(LUTCanvas):
 															 **kwargs)
 									w = 3
 								polys.append(poly(list(xy3),
-												  colour=wx.Colour(*RGBA),
+												  colour=wx.Colour(*intlist(RGBA)),
 												  width=w))
 								if i == 1:
 									xy3 = []
@@ -410,13 +412,13 @@ class GamutCanvas(LUTCanvas):
 				s = 2
 				w = 1.75
 			polys.append(plot.PolyMarker([(x, y)],
-										 colour=wx.Colour(*RGBA),
+										 colour=wx.Colour(*intlist(RGBA)),
 										 size=s,
 										 marker=marker,
 										 width=w))
 
 		xy_range = max(abs(min_x), abs(min_y)) + max(max_x, max_y)
-		if center:
+		if center or not self.last_draw:
 			self.axis_x = self.axis_y = (min(min_x, min_y), max(max_x, max_y))
 			self.spec_x = xy_range / step
 			self.spec_y = xy_range / step
@@ -431,7 +433,7 @@ class GamutCanvas(LUTCanvas):
 									  colour=wx.Colour(0x33, 0x33, 0x33),
 									  size=0)
 			graphics.objects.append(spacer)
-			if center:
+			if center or not self.last_draw:
 				boundingbox = spacer.boundingBox()
 				self.resetzoom(boundingbox)
 				self.center(boundingbox)
@@ -635,8 +637,7 @@ class GamutViewOptions(wx.Panel):
 	def __init__(self, *args, **kwargs):
 		wx.Panel.__init__(self, *args, **kwargs)
 		self.SetBackgroundColour(BGCOLOUR)
-		self.SetForegroundColour(FGCOLOUR)
-		self.sizer = wx.FlexGridSizer(0, 3, 4)
+		self.sizer = wx.FlexGridSizer(0, 3, 4, 0)
 		self.sizer.AddGrowableCol(0)
 		self.sizer.AddGrowableCol(2)
 		self.SetSizer(self.sizer)
@@ -968,8 +969,10 @@ class ProfileInfoFrame(LUTFrame):
 	
 		if len(args) < 3 and not "title" in kwargs:
 			kwargs["title"] = lang.getstr("profile.info")
+		if not "name" in kwargs:
+			kwargs["name"] = "profile_info"
 		
-		wx.Frame.__init__(self, *args, **kwargs)
+		BaseFrame.__init__(self, *args, **kwargs)
 		
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16],
 											 appname + "-profile-info"))
@@ -1191,8 +1194,8 @@ class ProfileInfoFrame(LUTFrame):
 			".icc": self.drop_handler,
 			".icm": self.drop_handler
 		}
-		droptarget = FileDrop(self, drophandlers)
-		self.client.SetDropTarget(droptarget)
+		self.droptarget = FileDrop(self, drophandlers)
+		self.client.SetDropTarget(self.droptarget)
 		droptarget = FileDrop(self, drophandlers)
 		self.grid.SetDropTarget(droptarget)
 
@@ -1222,6 +1225,7 @@ class ProfileInfoFrame(LUTFrame):
 
 		children = self.GetAllChildren()
 
+		self.Bind(wx.EVT_KEY_DOWN, self.key_handler)
 		for child in children:
 			if isinstance(child, wx.Choice):
 				child.SetMaxFontSize(11)
@@ -1253,9 +1257,9 @@ class ProfileInfoFrame(LUTFrame):
 			self.client.SetEnableGrid(False)
 			self.client.SetEnablePointLabel(True)
 			if ("vcgt" in self.profile.tags or
-				("rTRC" in self.profile.tags and
-				 "gTRC" in self.profile.tags and
-				 "bTRC" in self.profile.tags) or
+				(self.rTRC and
+				 self.gTRC and
+				 self.bTRC) or
 				(("B2A0" in self.profile.tags or
 				  "A2B0" in self.profile.tags) and
 				 self.profile.colorSpace == "RGB")):
@@ -1282,9 +1286,9 @@ class ProfileInfoFrame(LUTFrame):
 				self.DrawCanvas(reset=reset)
 				return
 		self.profile = profile
-		self.rTRC = profile.tags.get("rTRC")
-		self.gTRC = profile.tags.get("gTRC")
-		self.bTRC = profile.tags.get("bTRC")
+		self.rTRC = profile.tags.get("rTRC", profile.tags.get("kTRC"))
+		self.gTRC = profile.tags.get("gTRC", profile.tags.get("kTRC"))
+		self.bTRC = profile.tags.get("bTRC", profile.tags.get("kTRC"))
 		self.trc = None
 		
 		self.gamut_view_options.direction_select.Show("B2A0" in
@@ -1295,11 +1299,11 @@ class ProfileInfoFrame(LUTFrame):
 													 "A2B0" in profile.tags)
 		self.gamut_view_options.toggle_clut.Show("B2A0" in profile.tags or
 												 "A2B0" in profile.tags)
-		self.gamut_view_options.toggle_clut.Enable(isinstance(profile.tags.get("rTRC"),
+		self.gamut_view_options.toggle_clut.Enable(isinstance(self.rTRC,
 															  ICCP.CurveType) and
-												   isinstance(profile.tags.get("gTRC"),
+												   isinstance(self.gTRC,
 															  ICCP.CurveType) and
-												   isinstance(profile.tags.get("bTRC"),
+												   isinstance(self.bTRC,
 															  ICCP.CurveType))
 		self.toggle_clut.SetValue("B2A0" in profile.tags or
 								  "A2B0" in profile.tags)
@@ -1309,8 +1313,7 @@ class ProfileInfoFrame(LUTFrame):
 		choice = []
 		info = profile.get_info()
 		self.client.errors = []
-		if (("rTRC" in self.profile.tags and "gTRC" in self.profile.tags and
-			 "bTRC" in self.profile.tags) or
+		if ((self.rTRC and self.gTRC and self.bTRC) or
 			(("B2A0" in self.profile.tags or "A2B0" in self.profile.tags) and
 			 self.profile.colorSpace == "RGB")):
 			# vcgt needs to be in here for compatibility with LUTFrame
@@ -1318,7 +1321,7 @@ class ProfileInfoFrame(LUTFrame):
 			try:
 				self.lookup_tone_response_curves()
 			except Exception, exception:
-				show_result_dialog(exception, self)
+				wx.CallAfter(show_result_dialog, exception, self)
 			else:
 				choice.append(lang.getstr("[rgb]TRC"))
 		choice.append(lang.getstr("gamut"))
@@ -1646,6 +1649,27 @@ class ProfileInfoFrame(LUTFrame):
 		if not reset:
 			wx.CallAfter(self.client.center)
 		self.Thaw()
+
+	def get_commands(self):
+		return self.get_common_commands() + ["profile-info [filename]",
+											 "load <filename>"]
+
+	def process_data(self, data):
+		if (data[0] == "profile-info" and
+			len(data) < 3) or (data[0] == "load" and len(data) == 2):
+			if self.IsIconized():
+				self.Restore()
+			self.Raise()
+			if len(data) == 2:
+				path = data[1]
+				if not os.path.isfile(path) and not os.path.isabs(path):
+					path = get_data_path(path)
+				if not path:
+					return "fail"
+				else:
+					self.droptarget.OnDropFiles(0, 0, [path])
+			return "ok"
+		return "invalid"
 	
 	def redraw(self):
 		if self.plot_mode_select.GetSelection() == self.plot_mode_select.GetCount() - 1 and self.client.last_draw:
@@ -1798,7 +1822,7 @@ class ProfileInfoFrame(LUTFrame):
 									  wargs=(profile_paths, intent, direction,
 											 order, False),
 									  progress_msg=lang.getstr("gamut.view.create"),
-									  continue_next=x3d)
+									  continue_next=x3d, fancy=False)
 	
 	def view_3d_consumer(self, result, colorspace, filename, vrmlpath, x3d,
 						 x3dpath, html):
@@ -1857,24 +1881,30 @@ class ProfileInfoFrame(LUTFrame):
 		self.view_3d(None)
 
 
-class ProfileInfoViewer(wx.App):
-
-	def OnInit(self):
-		check_set_argyll_bin()
-		self.frame = ProfileInfoFrame(None, -1)
-		return True
-
-
-def main(profile=None):
-	config.initcfg()
+def main():
+	config.initcfg("profile-info")
 	lang.init()
 	lang.update_defaults()
-	app = ProfileInfoViewer(0)
-	display_no = get_argyll_display_number(app.frame.get_display()[1])
-	app.frame.LoadProfile(profile or get_display_profile(display_no))
-	app.frame.Show()
+	app = BaseApp(0)
+	check_set_argyll_bin()
+	app.TopWindow = ProfileInfoFrame(None, -1)
+	if sys.platform == "darwin":
+		app.TopWindow.init_menubar()
+	app.TopWindow.listen()
+	display_no = get_argyll_display_number(app.TopWindow.get_display()[1])
+	for arg in sys.argv[1:]:
+		if os.path.isfile(arg):
+			app.TopWindow.drop_handler(safe_unicode(arg))
+			break
+	else:
+		app.TopWindow.LoadProfile(get_display_profile(display_no))
+	app.TopWindow.Show()
 	app.MainLoop()
-	writecfg()
+	writecfg(module="profile-info", options=("3d.format",
+											 "last_cal_or_icc_path",
+											 "last_icc_path",
+											 "position.profile_info",
+											 "size.profile_info"))
 
 if __name__ == "__main__":
-	main(*sys.argv[max(len(sys.argv) - 1, 1):])
+	main()
