@@ -15,7 +15,7 @@ import colormath
 import config
 import localization as lang
 import worker
-from wxwindows import BaseFrame, FileDrop, InfoDialog, wx
+from wxwindows import BaseApp, BaseFrame, FileDrop, InfoDialog, wx
 import floatspin
 import xh_floatspin
 
@@ -30,9 +30,16 @@ class SynthICCFrame(BaseFrame):
 		self.res = xrc.XmlResource(get_data_path(os.path.join("xrc", 
 															  "synthicc.xrc")))
 		self.res.InsertHandler(xh_floatspin.FloatSpinCtrlXmlHandler())
-		pre = wx.PreFrame()
-		self.res.LoadOnFrame(pre, parent, "synthiccframe")
-		self.PostCreate(pre)
+		if hasattr(wx, "PreFrame"):
+			# Classic
+			pre = wx.PreFrame()
+			self.res.LoadOnFrame(pre, parent, "synthiccframe")
+			self.PostCreate(pre)
+		else:
+			# Phoenix
+			wx.Frame.__init__(self)
+			self.res.LoadFrame(self, parent, "synthiccframe")
+		self.init()
 		self.Bind(wx.EVT_CLOSE, self.OnClose)
 		
 		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16],
@@ -123,30 +130,41 @@ class SynthICCFrame(BaseFrame):
 			setcfg("position.synthiccframe.y", y)
 			setcfg("size.synthiccframe.w", self.GetSize()[0])
 			setcfg("size.synthiccframe.h", self.GetSize()[1])
-		config.writecfg()
+		if self.Parent:
+			config.writecfg()
+		else:
+			config.writecfg(module="synthprofile",
+							options=("synthprofile.", "last_icc_path",
+									 "position.synthiccframe",
+									 "size.synthiccframe"))
 		if event:
 			event.Skip()
 	
 	def black_luminance_ctrl_handler(self, event):
 		v = self.black_luminance_ctrl.GetValue()
 		white_Y = getcfg("synthprofile.luminance")
-		min_Y = (1 / 65535.0) * 100
-		increment = (1 / 65535.0) * white_Y
-		if increment < min_Y:
-			increment = min_Y * (white_Y / 100.0)
-		min_inc = 1.0 / (10.0 ** self.black_luminance_ctrl.GetDigits())
-		if increment < min_inc:
-			increment = min_inc
-		self.black_luminance_ctrl.SetIncrement(increment)
-		fmt = "%%.%if" % self.black_luminance_ctrl.GetDigits()
-		if fmt % v > fmt % 0 and fmt % v < fmt % increment:
+		if v >= white_Y * .9:
 			if event:
-				v = increment
-			else:
+				wx.Bell()
+			v = white_Y * .9
+		if event:
+			min_Y = (1 / 65535.0) * 100
+			increment = (1 / 65535.0) * white_Y
+			if increment < min_Y:
+				increment = min_Y * (white_Y / 100.0)
+			min_inc = 1.0 / (10.0 ** self.black_luminance_ctrl.GetDigits())
+			if increment < min_inc:
+				increment = min_inc
+			self.black_luminance_ctrl.SetIncrement(increment)
+			fmt = "%%.%if" % self.black_luminance_ctrl.GetDigits()
+			if fmt % v > fmt % 0 and fmt % v < fmt % increment:
+				if event:
+					v = increment
+				else:
+					v = 0
+			elif fmt % v == fmt % 0:
 				v = 0
-		elif fmt % v == fmt % 0:
-			v = 0
-		v = round(v / increment) * increment
+			v = round(v / increment) * increment
 		self.black_luminance_ctrl.SetValue(v)
 
 		old = getcfg("synthprofile.black_luminance")
@@ -159,6 +177,9 @@ class SynthICCFrame(BaseFrame):
 			self.Freeze()
 			i = self.trc_ctrl.GetSelection()
 			self.trc_gamma_type_ctrl.Show(i in (0, 4) and bool(v))
+			if not v:
+				self.bpc_ctrl.SetValue(False)
+			self.bpc_ctrl.Enable(bool(v))
 			black_output_offset_show = (i in (0, 4) and bool(v))
 			self.black_output_offset_label.Show(black_output_offset_show)
 			self.black_output_offset_ctrl.Show(black_output_offset_show)
@@ -234,7 +255,11 @@ class SynthICCFrame(BaseFrame):
 		except (IOError, ICCP.ICCProfileInvalidError), exception:
 			show_result_dialog(Error(lang.getstr("profile.invalid")), self)
 		else:
-			if (profile.colorSpace != "RGB" or
+			if profile.version >= 4:
+				show_result_dialog(Error(lang.getstr("profile.iccv4.unsupported")),
+								   self)
+				return
+			if (profile.colorSpace not in ("RGB", "GRAY") or
 				profile.connectionColorSpace not in ("Lab", "XYZ")):
 				show_result_dialog(Error(lang.getstr("profile.unsupported",
 													 (profile.profileClass,
@@ -245,10 +270,17 @@ class SynthICCFrame(BaseFrame):
 			for i in xrange(256):
 				rgb.append((1.0 / 255 * i, 1.0 / 255 * i, 1.0 / 255 * i))
 			try:
-				colors = self.worker.xicclu(profile, rgb, intent="a")
+				colors = self.worker.xicclu(profile, rgb, intent="a", pcs="x")
 			except Exception, exception:
 				show_result_dialog(exception, self)
 			else:
+				self.panel.Freeze()
+				for ctrl, value in [(self.colorspace_rgb_ctrl,
+									 profile.colorSpace == "RGB"),
+									(self.colorspace_gray_ctrl,
+									 profile.colorSpace == "GRAY")]:
+					ctrl.SetValue(value)
+				self.colorspace_ctrl_handler(None)
 				if "lumi" in profile.tags:
 					luminance = profile.tags.lumi.Y
 				else:
@@ -268,7 +300,7 @@ class SynthICCFrame(BaseFrame):
 								(color, component)).SetValue(xyY[j])
 				self.parse_xy(None)
 				self.black_XYZ_ctrl_handler(None)
-				trc = ICCP.CurveType()
+				trc = ICCP.CurveType(profile=profile)
 				for XYZ in colors[5:]:
 					trc.append(XYZ[1] / colors[0][1] * 65535)
 				transfer_function = trc.get_transfer_function()
@@ -281,6 +313,7 @@ class SynthICCFrame(BaseFrame):
 				self.set_trc(round(gamma, 2))
 				setcfg("synthprofile.trc_gamma_type", "g")
 				self.trc_gamma_type_ctrl.SetSelection(self.trc_gamma_types_ba["g"])
+				self.panel.Thaw()
 	
 	def enable_save_as_btn(self):
 		self.save_as_btn.Enable(bool(self.get_XYZ()))
@@ -408,6 +441,27 @@ class SynthICCFrame(BaseFrame):
 			self.set_trc(gamma)
 			self._updating_ctrls = False
 
+	def get_commands(self):
+		return self.get_common_commands() + ["synthprofile [filename]",
+											 "load <filename>"]
+
+	def process_data(self, data):
+		if (data[0] == "synthprofile" and
+			len(data) < 3) or (data[0] == "load" and len(data) == 2):
+			if self.IsIconized():
+				self.Restore()
+			self.Raise()
+			if len(data) == 2:
+				path = data[1]
+				if not os.path.isfile(path) and not os.path.isabs(path):
+					path = get_data_path(path)
+				if not path:
+					return "fail"
+				else:
+					self.droptarget.OnDropFiles(0, 0, [path])
+			return "ok"
+		return "invalid"
+
 	def set_trc(self, gamma):
 		if gamma == -1023:
 			# DICOM
@@ -425,9 +479,12 @@ class SynthICCFrame(BaseFrame):
 		elif gamma == -240:
 			# SMPTE 240M
 			self.trc_ctrl.SetSelection(5)
+		elif gamma == -2084:
+			# SMPTE 2084
+			self.trc_ctrl.SetSelection(6)
 		elif gamma == -2.4:
 			# sRGB
-			self.trc_ctrl.SetSelection(6)
+			self.trc_ctrl.SetSelection(7)
 		else:
 			# Gamma
 			self.trc_ctrl.SetSelection(0)
@@ -452,26 +509,22 @@ class SynthICCFrame(BaseFrame):
 			gamma = 2.2
 			self.trc_gamma_ctrl.Value = str(gamma)
 		white = XYZ["wX"], XYZ["wY"], XYZ["wZ"]
-		black = colormath.adapt(XYZ["kX"], XYZ["kY"], XYZ["kZ"], white)
-		if self.trc_ctrl.GetSelection() == 0:
-			# Gamma
+		if self.trc_ctrl.GetSelection() in (0, 1, 4, 6):
+			# 0 = Gamma
+			# 1 = DICOM - trc set here is not actually used in the end
+			# 4 = Rec. 1886 - trc set here is only used if black = 0
+			# 6 = SMPTE 2084 - trc set here is not actually used in the end
 			trc = gamma
-		elif self.trc_ctrl.GetSelection() == 1:
-			# DICOM - gamma set here is not actually used
-			trc = 2.2
 		elif self.trc_ctrl.GetSelection() == 2:
 			# L*
 			trc = -3.0
 		elif self.trc_ctrl.GetSelection() == 3:
 			# Rec. 709
 			trc = -709
-		elif self.trc_ctrl.GetSelection() == 4:
-			# Rec. 1886 - gamma set here is not actually used
-			trc = 2.2
 		elif self.trc_ctrl.GetSelection() == 5:
 			# SMPTE 240M
 			trc = -240
-		elif self.trc_ctrl.GetSelection() == 6:
+		elif self.trc_ctrl.GetSelection() == 7:
 			# sRGB
 			trc = -2.4
 		defaultDir, defaultFile = get_verified_path("last_icc_path")
@@ -485,6 +538,11 @@ class SynthICCFrame(BaseFrame):
 											   trc,
 											   defaultFile,
 											   getcfg("copyright"))
+			black = colormath.adapt(XYZ["kX"], XYZ["kY"], XYZ["kZ"], white)
+			profile.tags.rTRC = ICCP.CurveType(profile=profile)
+			profile.tags.gTRC = ICCP.CurveType(profile=profile)
+			profile.tags.bTRC = ICCP.CurveType(profile=profile)
+			channels = "rgb"
 		else:
 			# Grayscale profile
 			profile = ICCP.ICCProfile()
@@ -494,43 +552,72 @@ class SynthICCFrame(BaseFrame):
 			(profile.tags.wtpt.X,
 			 profile.tags.wtpt.Y,
 			 profile.tags.wtpt.Z) = (XYZ["wX"], XYZ["wY"], XYZ["wZ"])
-		profile.tags.rTRC = ICCP.CurveType()
-		profile.tags.gTRC = ICCP.CurveType()
-		profile.tags.bTRC = ICCP.CurveType()
+			black = [XYZ["wY"] * (getcfg("synthprofile.black_luminance") /
+								  getcfg("synthprofile.luminance"))] * 3
+			profile.tags.kTRC = ICCP.CurveType(profile=profile)
+			channels = "k"
 		if self.trc_ctrl.GetSelection() == 1:
 			# DICOM
 			# Absolute luminance values!
 			try:
-				profile.set_dicom_trc([v * getcfg("synthprofile.luminance")
-									   for v in black],
-									  getcfg("synthprofile.luminance"))
+				if self.colorspace_rgb_ctrl.Value:
+					# Color profile
+					profile.set_dicom_trc([v * getcfg("synthprofile.luminance")
+										   for v in black],
+										  getcfg("synthprofile.luminance"))
+				else:
+					# Grayscale profile
+					profile.tags.kTRC.set_dicom_trc(getcfg("synthprofile.black_luminance"),
+													getcfg("synthprofile.luminance"))
 			except ValueError, exception:
 				show_result_dialog(exception, self)
 				return
-		elif self.trc_ctrl.GetSelection() in (0, 4) and black != (0, 0, 0):
+		elif self.trc_ctrl.GetSelection() in (0, 4) and black != [0, 0, 0]:
 			# Gamma with output offset or Rec. 1886-like
 			outoffset = getcfg("synthprofile.trc_output_offset")
-			profile.set_bt1886_trc(black, outoffset, gamma,
-								   getcfg("synthprofile.trc_gamma_type"))
-		elif black != (0, 0, 0):
-			rXYZ = profile.tags.rXYZ.values()
-			gXYZ = profile.tags.gXYZ.values()
-			bXYZ = profile.tags.bXYZ.values()
-			mtx = colormath.Matrix3x3([[rXYZ[0], gXYZ[0], bXYZ[0]],
-									   [rXYZ[1], gXYZ[1], bXYZ[1]],
-									   [rXYZ[2], gXYZ[2], bXYZ[2]]])
-			rgbblack = mtx.inverted() * black
+			if self.colorspace_rgb_ctrl.Value:
+				# Color profile
+				profile.set_bt1886_trc(black, outoffset, gamma,
+									   getcfg("synthprofile.trc_gamma_type"))
+			else:
+				# Grayscale profile
+				profile.tags.kTRC.set_bt1886_trc(black[1], outoffset, gamma,
+												 getcfg("synthprofile.trc_gamma_type"))
+		elif self.trc_ctrl.GetSelection() == 6:
+			# SMPTE 2084
+			if self.colorspace_rgb_ctrl.Value:
+				# Color profile
+				profile.set_smpte2084_trc([v * getcfg("synthprofile.luminance")
+										   for v in black],
+										  getcfg("synthprofile.luminance"))
+			else:
+				# Grayscale profile
+				profile.tags.kTRC.set_smpte2084_trc(getcfg("synthprofile.black_luminance"),
+													getcfg("synthprofile.luminance"))
+		elif black != [0, 0, 0]:
+			if self.colorspace_rgb_ctrl.Value:
+				# Color profile
+				rXYZ = profile.tags.rXYZ.values()
+				gXYZ = profile.tags.gXYZ.values()
+				bXYZ = profile.tags.bXYZ.values()
+				mtx = colormath.Matrix3x3([[rXYZ[0], gXYZ[0], bXYZ[0]],
+										   [rXYZ[1], gXYZ[1], bXYZ[1]],
+										   [rXYZ[2], gXYZ[2], bXYZ[2]]])
+				rgbblack = mtx.inverted() * black
+			else:
+				# Grayscale profile
+				rgbblack = black
 			# Optimize for uInt16Number encoding
 			rgbblack = [round(max(v, 0) * 65535) / 65535 for v in rgbblack]
-			for i, channel in enumerate("rgb"):
+			for i, channel in enumerate(channels):
 				TRC = profile.tags["%sTRC" % channel]
 				TRC.set_trc(trc, 1024, vmin=rgbblack[i] * 65535)
 		else:
-			for channel in "rgb":
+			for channel in channels:
 				profile.tags["%sTRC" % channel].set_trc(trc, 1)
-		if not self.colorspace_rgb_ctrl.Value:
-			# Grayscale profile
-			profile.tags.kTRC = profile.tags.gTRC
+		if black != [0, 0, 0] and self.bpc_ctrl.Value:
+			for channel in channels:
+				profile.tags["%sTRC" % channel].apply_bpc()
 		for tagname in ("lumi", "bkpt"):
 			if tagname == "lumi":
 				# Absolute
@@ -562,7 +649,6 @@ class SynthICCFrame(BaseFrame):
 								   self)
 				return
 			setcfg("last_icc_path", path)
-			config.writecfg()
 			profile.setDescription(os.path.splitext(os.path.basename(path))[0])
 			profile.calculateID()
 			try:
@@ -593,14 +679,12 @@ class SynthICCFrame(BaseFrame):
 			setcfg("synthprofile.trc_gamma", 2.4)
 			setcfg("synthprofile.trc_gamma_type", "G")
 			setcfg("synthprofile.trc_output_offset", 0.0)
-			config.writecfg()
 		if not self._updating_ctrls:
 			self.update_trc_controls()
 
 	def trc_gamma_type_ctrl_handler(self, event):
 		setcfg("synthprofile.trc_gamma_type",
 			   self.trc_gamma_types_ab[self.trc_gamma_type_ctrl.GetSelection()])
-		config.writecfg()
 		self.update_trc_control()
 	
 	def trc_gamma_ctrl_handler(self, event):
@@ -618,7 +702,6 @@ class SynthICCFrame(BaseFrame):
 					self.trc_gamma_ctrl.SetValue(str(v))
 				setcfg("synthprofile.trc_gamma",
 					   float(self.trc_gamma_ctrl.GetValue()))
-				config.writecfg()
 				self.preset_ctrl.SetSelection(0)
 				self.update_trc_control()
 		event.Skip()
@@ -666,12 +749,16 @@ class SynthICCFrame(BaseFrame):
 
 
 def main():
-	config.initcfg()
+	config.initcfg("synthprofile")
 	lang.init()
 	lang.update_defaults()
-	app = wx.App(0)
-	app.synthiccframe = SynthICCFrame()
-	app.synthiccframe.Show()
+	app = BaseApp(0)
+	app.TopWindow = SynthICCFrame()
+	if sys.platform == "darwin":
+		app.TopWindow.init_menubar()
+	app.TopWindow.listen()
+	app.process_argv(1)
+	app.TopWindow.Show()
 	app.MainLoop()
 
 if __name__ == "__main__":
