@@ -19,9 +19,10 @@ from config import (getbitmap, getcfg, geticon, get_data_path, get_icon_bundle,
 from log import get_file_logger, safe_print
 from meta import name as appname
 from options import debug, test, verbose
-from wxwindows import (BitmapBackgroundPanel, CustomCheckBox, CustomGrid,
-					   FlatShadedButton, numpad_keycodes)
+from wxwindows import (BaseApp, BaseFrame, BitmapBackgroundPanel, CustomCheckBox,
+					   CustomGrid, FlatShadedButton, numpad_keycodes)
 import CGATS
+import audio
 import colormath
 import config
 import localization as lang
@@ -30,15 +31,16 @@ BGCOLOUR = wx.Colour(0x33, 0x33, 0x33)
 FGCOLOUR = wx.Colour(0x99, 0x99, 0x99)
 
 
-class UntetheredFrame(wx.Frame):
+class UntetheredFrame(BaseFrame):
 
 	def __init__(self, parent=None, handler=None,
 				 keyhandler=None, start_timer=True):
-		wx.Frame.__init__(self, parent, wx.ID_ANY,
+		BaseFrame.__init__(self, parent, wx.ID_ANY,
 						  lang.getstr("measurement.untethered"),
-						  style=wx.DEFAULT_FRAME_STYLE)
+						  style=wx.DEFAULT_FRAME_STYLE,
+						  name="untetheredframe")
 		self.SetIcons(get_icon_bundle([256, 48, 32, 16], appname))
-		self.sizer = wx.FlexGridSizer(2, 1)
+		self.sizer = wx.FlexGridSizer(2, 1, 0, 0)
 		self.sizer.AddGrowableCol(0)
 		self.sizer.AddGrowableRow(0)
 		self.sizer.AddGrowableRow(1)
@@ -98,17 +100,14 @@ class UntetheredFrame(wx.Frame):
 		sizer = wx.BoxSizer(wx.HORIZONTAL)
 		self.measure_btn = FlatShadedButton(self.panel,
 											bitmap=geticon(10, "play"),
-											label=" " + lang.getstr("measure"),
+											label=lang.getstr("measure"),
 											fgcolour=FGCOLOUR)
 		self.measure_btn.Bind(wx.EVT_BUTTON, self.measure_btn_handler)
 		sizer.Add(self.measure_btn, 0, wx.RIGHT, border=6)
 		# Sound when measuring
 		# Needs to be stereo!
-		try:
-			self.measurement_sound = wx.Sound(get_data_path("beep.wav") or "")
-			self.commit_sound = wx.Sound(get_data_path("camera_shutter.wav") or "")
-		except NotImplementedError:
-			pass
+		self.measurement_sound = audio.Sound(get_data_path("beep.wav"))
+		self.commit_sound = audio.Sound(get_data_path("camera_shutter.wav"))
 		if getcfg("measurement.play_sound"):
 			bitmap = geticon(16, "sound_volume_full")
 		else:
@@ -445,10 +444,8 @@ class UntetheredFrame(wx.Frame):
 			self.last_error = txt
 		if "Result is XYZ:" in txt:
 			self.last_error = None
-			if (getattr(self, "measurement_sound", None) and
-				getcfg("measurement.play_sound") and
-				self.measurement_sound.IsOk()):
-				self.measurement_sound.Play(wx.SOUND_ASYNC)
+			if getcfg("measurement.play_sound"):
+				self.measurement_sound.safe_play()
 			# Result is XYZ: d.dddddd d.dddddd d.dddddd, D50 Lab: d.dddddd d.dddddd d.dddddd
 			XYZ = re.search("XYZ:\s+(-?\d+(?:\.\d+)?)\s+(-?\d+(?:\.\d+)?)\s+(-?\d+(?:\.\d+)?)",
 							txt)
@@ -478,10 +475,8 @@ class UntetheredFrame(wx.Frame):
 				 abs(delta["C"]) < getcfg("untethered.max_delta.chroma"))):
 				self.measure_count += 1
 				if self.measure_count == 2:
-					if (getattr(self, "commit_sound", None) and
-						getcfg("measurement.play_sound") and
-						self.commit_sound.IsOk()):
-						self.commit_sound.Play(wx.SOUND_ASYNC)
+					if getcfg("measurement.play_sound"):
+						self.commit_sound.safe_play()
 					self.measure_count = 0
 					# Reset row label
 					self.grid.SetRowLabelValue(self.index, "%i" % (self.index + 1))
@@ -698,8 +693,8 @@ if __name__ == "__main__":
 	print "untethered.max_delta.chroma", getcfg("untethered.max_delta.chroma")
 	lang.init()
 	lang.update_defaults()
-	app = wx.App(0)
-	frame = UntetheredFrame(start_timer=False)
+	app = BaseApp(0)
+	app.TopWindow = UntetheredFrame(start_timer=False)
 	testchart = getcfg("testchart.file")
 	if os.path.splitext(testchart)[1].lower() in (".icc", ".icm"):
 		try:
@@ -707,9 +702,9 @@ if __name__ == "__main__":
 		except:
 			pass
 	try:
-		frame.cgats = CGATS.CGATS(testchart)
+		app.TopWindow.cgats = CGATS.CGATS(testchart)
 	except:
-		frame.cgats = CGATS.CGATS("""TI1    
+		app.TopWindow.cgats = CGATS.CGATS("""TI1    
 BEGIN_DATA_FORMAT
 SAMPLE_ID RGB_R RGB_G RGB_B XYZ_X XYZ_Y XYZ_Z
 END_DATA_FORMAT
@@ -717,10 +712,10 @@ BEGIN_DATA
 1 0 0 0 0 0 0
 END_DATA
 """)
-	frame.worker = Worker()
-	frame.worker.progress_wnd = frame
-	frame.Show()
-	files = Files([frame.worker, frame])
+	app.TopWindow.worker = Worker()
+	app.TopWindow.worker.progress_wnd = app.TopWindow
+	app.TopWindow.Show()
+	files = Files([app.TopWindow.worker, app.TopWindow])
 	def test(bytes=None):
 		print "*** Received %r" % bytes
 		menu = r"""Place instrument on spot to be measured,
@@ -732,8 +727,8 @@ Hit ESC or Q to exit, any other key to take a reading:"""
 		if not bytes:
 			txt = menu
 		elif bytes == " ":
-			i = frame.index
-			row = frame.cgats[0].DATA[i]
+			i = app.TopWindow.index
+			row = app.TopWindow.cgats[0].DATA[i]
 			txt = ["""
  Result is XYZ: %.6f %.6f %.6f
 
@@ -756,13 +751,13 @@ or place on the white calibration reference,
 								 row.XYZ_Y,
 								 row.XYZ_Z)][random.choice([0, 1])]
 		elif bytes in ("Q", "q"):
-			wx.CallAfter(frame.Close)
+			wx.CallAfter(app.TopWindow.Close)
 			return
 		else:
 			return
 		for line in txt.split("\n"):
 			sleep(.03125)
-			if frame:
+			if app.TopWindow:
 				wx.CallAfter(files.write, line)
 				print line
 	start_new_thread(test, tuple())
