@@ -10,13 +10,84 @@ from util_str import safe_unicode
 from worker import Worker, show_result_dialog
 from wxaddons import wx
 from wxfixes import GenBitmapButton as BitmapButton
-from wxwindows import FileDrop
+from wxwindows import BaseApp, BaseFrame, FileDrop
 import config
 import localization as lang
 import x3dom
 
 
-def main(vrmlpath=None):
+class VRML2X3DFrame(BaseFrame):
+
+	def __init__(self, html, embed, view, force, cache):
+		BaseFrame.__init__(self, None, wx.ID_ANY,
+						   lang.getstr("vrml_to_x3d_converter"),
+						   style=wx.DEFAULT_FRAME_STYLE & ~(wx.MAXIMIZE_BOX |
+															wx.RESIZE_BORDER),
+						   name="vrml2x3dframe")
+		self.SetIcons(config.get_icon_bundle([256, 48, 32, 16],
+											  appname +
+											  "-VRML-to-X3D-converter"))
+		self.cache = cache
+		self.embed = embed
+		self.force = force
+		self.html = html
+		self.worker = Worker(self)
+		sizer = wx.BoxSizer(wx.HORIZONTAL)
+		self.SetSizer(sizer)
+		panel = wx.Panel(self)
+		sizer.Add(panel)
+		panelsizer = wx.BoxSizer(wx.HORIZONTAL)
+		panel.SetSizer(panelsizer)
+		self.btn = BitmapButton(panel, wx.ID_ANY,
+								config.geticon(256, "document-open"), 
+								style=wx.NO_BORDER)
+		self.btn.SetToolTipString(lang.getstr("file.select"))
+		self.btn.Bind(wx.EVT_BUTTON, lambda event:
+									  vrmlfile2x3dfile(None,
+													   html=html,
+													   embed=embed,
+													   view=view,
+													   force=force,
+													   cache=cache,
+													   worker=self.worker))
+		self.droptarget = FileDrop(self)
+		vrml_drop_handler = lambda vrmlpath: vrmlfile2x3dfile(vrmlpath,
+															  html=html,
+															  embed=embed,
+															  view=view,
+															  force=force,
+															  cache=cache,
+															  worker=self.worker)
+		self.droptarget.drophandlers = {
+			".vrml": vrml_drop_handler,
+			".vrml.gz": vrml_drop_handler,
+			".wrl": vrml_drop_handler,
+			".wrl.gz": vrml_drop_handler,
+			".wrz": vrml_drop_handler
+		}
+		self.btn.SetDropTarget(self.droptarget)
+		panelsizer.Add(self.btn, flag=wx.ALL, border=12)
+		self.Fit()
+		self.SetMinSize(self.GetSize())
+		self.SetMaxSize(self.GetSize())
+
+	def get_commands(self):
+		return (self.get_common_commands() +
+			    ["VRML-to-X3D-converter [filename...]",
+				 "load <filename...>"])
+
+	def process_data(self, data):
+		if data[0] in ("VRML-to-X3D-converter", "load"):
+			if self.IsIconized():
+				self.Restore()
+			self.Raise()
+			if len(data) > 1:
+				self.droptarget.OnDropFiles(0, 0, data[1:])
+			return "ok"
+		return "invalid"
+
+
+def main():
 	if "--help" in sys.argv[1:]:
 		safe_print("Usage: %s [--embed] [--no-gui] [--view] [FILE]" % sys.argv[0])
 		safe_print("Convert VRML file to X3D")
@@ -34,67 +105,28 @@ def main(vrmlpath=None):
 		safe_print("  --view       View the generated file (if --no-gui)")
 		safe_print("  FILE         Filename of VRML file to convert")
 		return
-	config.initcfg()
+	config.initcfg("%s-VRML-to-X3D-converter" % appname)
 	lang.init()
+	lang.update_defaults()
 	cache = not "--no-cache" in sys.argv[1:]
 	embed = "--embed" in sys.argv
 	force = "--force" in sys.argv
 	html = not "--no-html" in sys.argv[1:]
-	view = "--view" in sys.argv[1:]
 	if "--no-gui" in sys.argv[1:]:
-		vrmlfile2x3dfile(vrmlpath, html=html, embed=embed, view=view,
-						 force=force, cache=cache)
+		view = "--view" in sys.argv[1:]
+		for arg in sys.argv[1:]:
+			if os.path.isfile(arg):
+				vrmlfile2x3dfile(safe_unicode(arg), html=html, embed=embed,
+								 view=view, force=force, cache=cache)
 	else:
-		app = wx.App(0)
-		frame = wx.Frame(None, wx.ID_ANY, lang.getstr("vrml_to_x3d_converter"),
-						 style=wx.DEFAULT_FRAME_STYLE & ~(wx.MAXIMIZE_BOX |
-														  wx.RESIZE_BORDER))
-		frame.SetIcons(config.get_icon_bundle([256, 48, 32, 16],
-											  appname +
-											  "-VRML-to-X3D-converter"))
-		worker = Worker(frame)
-		sizer = wx.BoxSizer(wx.HORIZONTAL)
-		frame.SetSizer(sizer)
-		panel = wx.Panel(frame)
-		sizer.Add(panel)
-		panelsizer = wx.BoxSizer(wx.HORIZONTAL)
-		panel.SetSizer(panelsizer)
-		btn = BitmapButton(panel, wx.ID_ANY,
-						   config.geticon(256, "document-open"), 
-						   style=wx.NO_BORDER)
-		btn.SetToolTipString(lang.getstr("file.select"))
-		btn.Bind(wx.EVT_BUTTON, lambda event:
-								vrmlfile2x3dfile(None,
-												 html=html,
-												 embed=embed,
-												 view=True,
-												 force=force,
-												 cache=cache,
-												 worker=worker))
-		droptarget = FileDrop(frame)
-		vrml_drop_handler = lambda vrmlpath: vrmlfile2x3dfile(vrmlpath,
-															  html=html,
-															  embed=embed,
-															  view=True,
-															  force=force,
-															  cache=cache,
-															  worker=worker)
-		droptarget.drophandlers = {
-			".vrml": vrml_drop_handler,
-			".vrml.gz": vrml_drop_handler,
-			".wrl": vrml_drop_handler,
-			".wrl.gz": vrml_drop_handler,
-			".wrz": vrml_drop_handler
-		}
-		btn.SetDropTarget(droptarget)
-		panelsizer.Add(btn, flag=wx.ALL, border=12)
-		frame.Fit()
-		frame.SetMinSize(frame.GetSize())
-		frame.SetMaxSize(frame.GetSize())
-		frame.Show()
-		if vrmlpath:
-			wx.CallAfter(vrmlfile2x3dfile, vrmlpath, html=html, embed=embed,
-						 view=True, force=force, cache=cache, worker=worker)
+		view = not "--no-view" in sys.argv[1:]
+		app = BaseApp(0)
+		app.TopWindow = VRML2X3DFrame(html, embed, view, force, cache)
+		if sys.platform == "darwin":
+			app.TopWindow.init_menubar()
+		app.TopWindow.listen()
+		app.process_argv()
+		app.TopWindow.Show()
 		app.MainLoop()
 
 
@@ -111,7 +143,7 @@ def vrmlfile2x3dfile(vrmlpath=None, x3dpath=None, html=True, embed=False,
 				safe_print("%r is not a file." % vrmlpath)
 			sys.exit(1)
 		if not wx.GetApp():
-			app = wx.App(0)
+			app = BaseApp(0)
 		defaultDir, defaultFile = config.get_verified_path("last_vrml_path")
 		dlg = wx.FileDialog(None, lang.getstr("file.select"),
 							defaultDir=defaultDir, 
@@ -121,12 +153,13 @@ def vrmlfile2x3dfile(vrmlpath=None, x3dpath=None, html=True, embed=False,
 							style=wx.FD_OPEN | wx.FD_FILE_MUST_EXIST)
 		dlg.Center(wx.BOTH)
 		result = dlg.ShowModal()
-		if result != wx.ID_OK:
-			return
 		vrmlpath = dlg.GetPath()
 		dlg.Destroy()
+		if result != wx.ID_OK:
+			return
 		config.setcfg("last_vrml_path", vrmlpath)
-		config.writecfg()
+		config.writecfg(module="VRML-to-X3D-converter",
+						options=("last_vrml_path", ))
 	filename, ext = os.path.splitext(vrmlpath)
 	if x3dpath is None:
 		x3dpath = filename + ".x3d"
@@ -140,7 +173,7 @@ def vrmlfile2x3dfile(vrmlpath=None, x3dpath=None, html=True, embed=False,
 				safe_print("%r is not writable." % dirname)
 			sys.exit(1)
 		if not wx.GetApp():
-			app = wx.App(0)
+			app = BaseApp(0)
 		if x3dpath:
 			defaultDir, defaultFile = os.path.split(x3dpath)
 		else:
@@ -180,8 +213,9 @@ def vrmlfile2x3dfile(vrmlpath=None, x3dpath=None, html=True, embed=False,
 					 wargs=(vrmlpath, x3dpath, html, embed, force, cache,
 							worker),
 					 progress_title=lang.getstr("vrml_to_x3d_converter"),
+					 progress_start=1,
 					 resume=worker.progress_wnd and
-							worker.progress_wnd.IsShownOnScreen())
+							worker.progress_wnd.IsShownOnScreen(), fancy=False)
 	else:
 		result = x3dom.vrmlfile2x3dfile(vrmlpath, x3dpath, html, embed, force,
 										cache, None)
@@ -192,4 +226,4 @@ def vrmlfile2x3dfile(vrmlpath=None, x3dpath=None, html=True, embed=False,
 
 
 if __name__ == "__main__":
-	main(*sys.argv[max(len(sys.argv) - 1, 1):])
+	main()
