@@ -68,7 +68,7 @@ class Tag(object):
 			float(self.attributes.get("transparency",
 									  "0").strip()) not in (0.0, 1.0) and x3dom):
 			# Fix z-fighting in X3DOM renderer
-			markup += "<DepthMode readOnly='true'></DepthMode>"
+			markup.append("<DepthMode readOnly='true'></DepthMode>")
 		return "".join(markup)
 
 	def append_child(self, child):
@@ -199,7 +199,7 @@ class Tag(object):
 
 def _attrchk(attribute, token, tag, indent):
 	if attribute:
-		if verbose > 1 or debug:
+		if debug:
 			if tag.attributes.get(token):
 				safe_print(indent, "attribute %r %r" % (token,
 														tag.attributes[token]))
@@ -322,7 +322,7 @@ def get_vrml_axes(xlabel="X", ylabel="Y", zlabel="Z", offsetx=0,
 
 
 def safe_print(*args, **kwargs):
-	if verbose > 1 or debug:
+	if debug:
 		_safe_print(*args, **kwargs)
 
 
